unit uCommunication;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ComCtrls, ExtCtrls,
  FileUtil, Menus, laz.VirtualTrees, uFileTransfer, uMessages, uTCPSockets;

type
  TComType = (ctClient, ctServer, ctServerStressTest);
  TfCommunication = class(TForm)
    pgCommunication: TPageControl;
    tsFiles: TTabSheet;
    tsMessages: TTabSheet;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FTCPBase: TTCPBase;
    FFileTransfer: TfrFileTransfer;
    FMessages: TfrMessages;
    procedure DoOnChanged(Sender: TObject; AType: Byte);
    procedure DoOnSendFiles(Sender: TObject; AFiles: TStrings);
    procedure DoOnSendMessage(Sender: TObject; AMsg: String);
  public
    constructor Create(TheOwner: TControl; ATCPBase: TTCPBase); reintroduce;
  end;

var
  fCommunication: TfCommunication;

implementation

uses uMain, uFunctions;

{$R *.lfm}

constructor TfCommunication.Create(TheOwner: TControl; ATCPBase: TTCPBase);
begin
  inherited Create(TheOwner);
  FTCPBase := ATCPBase;
end;

procedure TfCommunication.FormCreate(Sender: TObject);
begin
  Self.Width := fMain.Width + 150;
  Self.Height := fMain.Height - 20;
  pgCommunication.ActivePageIndex := 0;

  FFileTransfer := TfrFileTransfer.Create(Self);
  FFileTransfer.Parent := tsFiles;
  FFileTransfer.Align := alClient;
  FFileTransfer.InitializeFrame(0);
  FFileTransfer.OnChanged := @DoOnChanged;
  FFileTransfer.OnSendFiles := @DoOnSendFiles;
  with fMain, FFileTransfer do
  begin
    FTCPBase.OnSentFile := @DoOnSentFile;
    FTCPBase.OnRecvFile := @DoOnRecvFile;
    FTCPBase.OnFileProgress := @DoOnFileProgress;
  end;

  FMessages := TfrMessages.Create(nil);
  FMessages.Parent := tsMessages;
  FMessages.Align := alClient;
  FMessages.InitializeFrame(0);
  FMessages.OnChanged := @DoOnChanged;
  FMessages.OnSendMessage := @DoOnSendMessage;
  with fMain, FMessages do
  begin
    FTCPBase.OnSentMessage := @DoOnSentMessage;
    FTCPBase.OnRecvMessage := @DoOnRecvMessage;
  end;
end;

procedure TfCommunication.FormDestroy(Sender: TObject);
begin
  FFileTransfer.FinalizeFrame;
  FFileTransfer.Free;
  FMessages.FinalizeFrame;
  FMessages.Free;
end;

procedure TfCommunication.DoOnChanged(Sender: TObject; AType: Byte);
begin
  if not Self.Visible then
  begin
    Self.Visible := True;
    pgCommunication.ActivePageIndex := AType;
  end;
end;

procedure TfCommunication.DoOnSendFiles(Sender: TObject; AFiles: TStrings);
var
  I: Integer;
  Node: PVirtualNode;
  Data: PFileData;
begin
  if not TTCPClient(FTCPBase).Connected then
  begin
    MessageDlg('Not connected!', mtInformation, [mbOk], 0);
    Exit;
  end;

  for I := 0 to AFiles.Count - 1 do
  begin
    if not TTCPClient(FTCPBase).Connected then
      Break;
    Node := FFileTransfer.VSTFiles.AddChild(nil);
    Data := FFileTransfer.VSTFiles.GetNodeData(Node);
    Data^.FFileName := AFiles[I];
    Data^.FProgress := 0;
    Data^.FProgressStr := 'Please wait...';
    Data^.FSizeStr := FormatSize(0);
    Data^.FSpeedStr := FormatSpeed(0);
    Data^.FRemainingStr := 'unknown';
    Data^.FTotalSize := FileSize(AFiles[I]);
    Data^.FTotalSizeStr := FormatSize(FileSize(AFiles[I]));
    TTCPClient(FTCPBase).SendFile(AFiles[I], PVirtualNode(Node));
    Application.ProcessMessages;
  end;
end;

procedure TfCommunication.DoOnSendMessage(Sender: TObject; AMsg: String);
begin
  if not TTCPClient(FTCPBase).Connected then
  begin
    MessageDlg('Not connected!', mtInformation, [mbOk], 0);
    Exit;
  end;
  TTCPClient(FTCPBase).SendMessage(AMsg, []);
end;

end.

