unit uMain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls,
  Menus, LCLType, LazFileUtils, laz.VirtualTrees, ComCtrls, ActnList, blcksock,
  uSettings, uTCPSockets;

const
  Title = 'Synapse client';
  Version = '2.0';
  SynTrunkRev = 267;
  ConfigFile = 'synapse_client_' + Version + '.xml';

type
  TWaitType = (wtStart, wtClose);
  TClientStatus = (csBeginSession, csConnected, csDisconnected, csEndSession);
  TfMain = class(TForm)
    acConnect: TAction;
    acDisconnect: TAction;
    acQuit: TAction;
    acOptions: TAction;
    acAbout: TAction;
    alMain: TActionList;
    imSB: TImageList;
    lbLog: TListBox;
    MenuItem1: TMenuItem;
    miDisconnectPU: TMenuItem;
    miOpen: TMenuItem;
    miShowErrorMessages: TMenuItem;
    miShow: TMenuItem;
    miSave: TMenuItem;
    mm: TMainMenu;
    miAbout: TMenuItem;
    miHelp: TMenuItem;
    miOptions: TMenuItem;
    miTools: TMenuItem;
    miDisconnect: TMenuItem;
    miConnect: TMenuItem;
    miConnection: TMenuItem;
    miQuit: TMenuItem;
    miFile: TMenuItem;
    miClear: TMenuItem;
    pmAction: TPopupMenu;
    pnMain: TPanel;
    pmLog: TPopupMenu;
    SD: TSaveDialog;
    sb: TStatusBar;
    sp: TSplitter;
    tmWait: TTimer;
    VSTCon: TLazVirtualStringTree;
    procedure acAboutExecute(Sender: TObject);
    procedure acConnectExecute(Sender: TObject);
    procedure acDisconnectExecute(Sender: TObject);
    procedure acQuitExecute(Sender: TObject);
    procedure acOptionsExecute(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure miClearClick(Sender: TObject);
    procedure miOpenClick(Sender: TObject);
    procedure miShowErrorMessagesClick(Sender: TObject);
    procedure miSaveClick(Sender: TObject);
    procedure pmLogPopup(Sender: TObject);
    procedure sbDrawPanel(StatusBar: TStatusBar; Panel: TStatusPanel;
      const Rect: TRect);
    procedure tmWaitTimer(Sender: TObject);
    procedure VSTConCompareNodes(Sender: TBaseVirtualTree; Node1,
      Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
    procedure VSTConDblClick(Sender: TObject);
    procedure VSTConFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
    procedure VSTConGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType; var CellText: String);
    procedure VSTConHeaderClick(Sender: TVTHeader; HitInfo: TVTHeaderHitInfo);
    procedure VSTConMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure VSTConPaintText(Sender: TBaseVirtualTree;
      const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
      TextType: TVSTTextType);
  private
    FTCPClient: TTCPClient;
    FClientStatus: TClientStatus;
    FSettings: TSettings;
    FByUser: Boolean;
    FSortCol: Integer;
    FSortDir: laz.VirtualTrees.TSortDirection;
    procedure Setup(AConData: TConData = nil);
    procedure AddLog(ALogMsg: String);
    procedure DoOnConnected(Sender: TObject; AConData: TConData; AConCnt: Integer = 0);
    procedure DoOnDisconnected(Sender: TObject; AConData: TConData; AConCnt: Integer = 0; AByUser: Boolean = False);
    procedure DoOnSocketError(Sender: TObject; AErrMsg: string; AErrCode: Integer; AConData: TConData);
    procedure DoOnInternalMessage(Sender: TObject; AMsg: String);
  public
    property TCPClient: TTCPClient read FTCPClient;
  end;

var
  fMain: TfMain;

implementation

uses uCommunication, uLogin, uOptions, uAbout;

{$R *.lfm}

type
  PData = ^TData;
  TData = record
    FIP: String;
    FHost: String;
    FLocalUser: String;
    FRemoteUser: String;
    FCC: TConnectedClient;
  end;

procedure TfMain.FormCreate(Sender: TObject);
var
  Path: String;
begin
  FSettings := TSettings.Create;
  Path := GetAppConfigDir(False);
  if not DirectoryExists(Path) then
    if not ForceDirectory(Path) then
      Path := GetTempDir(True);
  FSettings.FileName := Path  + ConfigFile;
  FSettings.Load;
  FTCPClient := TTCPClient.Create;
  FTCPClient.OnConnected := @DoOnConnected;
  FTCPClient.OnDisconnected := @DoOnDisconnected;
  FTCPClient.OnSocketError := @DoOnSocketError;
  fCommunication := TfCommunication.Create(Self, FTCPClient);
  Self.Width := 720;
  Self.Height := 450;
  Application.Title := Title + '  ' + Version;
  FSortCol := 0;
  FSortDir := laz.VirtualTrees.sdAscending;
end;

procedure TfMain.FormShow(Sender: TObject);
begin
  FClientStatus := csBeginSession;
  Setup;
end;

procedure TfMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := MessageDlg('Are you sure you wish to quit?', mtConfirmation, [mbYes, mbNo], 0) = mrYes;
  if not CanClose then
    Exit;
  if FTCPClient.Connected then
  begin
    CanClose := False;
    FClientStatus := csEndSession;
    Setup;
  end;
end;

procedure TfMain.FormDestroy(Sender: TObject);
begin
  FSettings.Free;
  FTCPClient.Free;
end;

procedure TfMain.tmWaitTimer(Sender: TObject);
begin
  case TWaitType(tmWait.Tag) of
    wtStart:
      begin
        tmWait.Enabled := False;
        acConnectExecute(acConnect);
      end;
    wtClose:
      begin
        if not FTCPClient.Connected then
        begin
          tmWait.Enabled := False;
          Application.Terminate;
        end
        else
          FTCPClient.Disconnect;
      end;
  end;
end;

procedure TfMain.sbDrawPanel(StatusBar: TStatusBar; Panel: TStatusPanel;
  const Rect: TRect);
var
  H, T, L: Integer;
begin
  with StatusBar.Canvas do
  begin
    H := StatusBar.Canvas.GetTextHeight('test');
    T := Rect.Top + (Rect.Bottom - Rect.Top - H) div 2;
    L := Rect.Left + 5;
    if Panel.Index = 0 then
    begin
      case sb.Tag of
        0: imSB.Draw(StatusBar.Canvas, L, T, 0);
        1: imSB.Draw(StatusBar.Canvas, L, T, 1);
        2: imSB.Draw(StatusBar.Canvas, L, T, 2);
      end;
      TextRect(Rect, L + ImSb.Width + 5, T, Panel.Text);
    end;
  end;
end;

procedure TfMain.Setup(AConData: TConData);
var
  ProxyType: String;
  ProxyStr: String;
  LogStr: String;
begin
   case FClientStatus of
    csBeginSession:
      begin
        Caption := Title + ' - disconnected';
        LogStr := 'Idle. Waiting for commands...';
        AddLog(LogStr);
        sb.Tag := 0;
        sb.Panels[0].Text := LogStr;
        sp.Visible := FSettings.ShowLog;
        acConnect.Enabled := True;
        acDisconnect.Enabled := False;
        pnMain.Visible := False;
        lbLog.Visible := FSettings.ShowLog;
        VSTCon.NodeDataSize := SizeOf(TData);
        tmWait.Tag := Ord(wtStart);
        tmWait.Enabled := True;
      end;
    csConnected:
      begin
        Caption := Title + ' - connected';
        if FSettings.ActiveConnection.FProxy.FProxyType <> ptNone then
        begin
          case FSettings.ActiveConnection.FProxy.FProxyType of
            ptHTML: ProxyType := 'HTML';
            ptSocks4: ProxyType := 'Socks4';
            ptSocks5: ProxyType := 'Socks5';
            else
              ProxyType := 'unknown';
          end;
          ProxyStr := '(via ' + FSettings.ActiveConnection.FProxy.FProxyIP + ' ' + ProxyType + ' proxy server)';
          LogStr := 'Connected to: ' + AConData.FRemoteIP + ProxyStr;
        end
        else
          LogStr := 'Connected to: ' + AConData.FRemoteIP;
        AddLog(LogStr + ' as user "' + AConData.FLocalUser + '".');
        sb.Tag := 1;
        sb.Panels[0].Text := LogStr;
        acConnect.Enabled := False;
        acDisconnect.Enabled := True;
        FByUser := False;
      end;
    csDisconnected:
      begin
        Caption := Title + ' - disconnected';
        if FByUser then
          LogStr := 'Disconnected by user'
        else
          LogStr := 'Disconnected';
        AddLog(LogStr + '.');
        pnMain.Visible := True;
        sb.Tag := 2;
        sb.Panels[0].Text := LogStr;
        acConnect.Enabled := True;
        acDisconnect.Enabled := False;
        if (not FByUser) then
          if MessageDlg('Connection lost. Do you wish to (re)conect?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
             tmWait.Enabled := True;
      end;
    csEndSession:
      begin
        Caption := 'Disconnecting from server. Please wait...';
        fCommunication.Close;
        fMain.Enabled := False;
        tmWait.Tag := Ord(wtClose);
        tmWait.Enabled := True;
      end;
  end;
  Application.ProcessMessages;
end;

{$Region 'TCPClient events'}
procedure TfMain.DoOnConnected(Sender: TObject; AConData: TConData;
  AConCnt: Integer);
var
  Node: PVirtualNode;
  Data: PData;
begin
  Node := VSTCon.AddChild(nil);
  Data := VSTCon.GetNodeData(Node);
  Data^.FLocalUser := AConData.FLocalUser;
  Data^.FRemoteUser := AConData.FRemoteUser;
  Data^.FIP := AConData.FRemoteIP;
  Data^.FHost := AConData.FRemoteHost;
  Data^.FCC := TConnectedClient(Sender);
  AConData.FData := PVirtualNode(Node);
  if VSTCon.FocusedNode = nil then
  begin
    VSTCon.FocusedNode := Node;
    VSTCon.Selected[Node] := True;
  end;
  VSTCon.SortTree(FSortCol, FSortDir);
  if FClientStatus <> csEndSession then
    FClientStatus := csConnected;
  Setup(AConData);
end;

procedure TfMain.DoOnDisconnected(Sender: TObject; AConData: TConData;
  AConCnt: Integer; AByUser: Boolean = False);
var
  Node: PVirtualNode;
  Data: PData;
begin
  Node := PVirtualNode(AConData.FData);
  if Node <> nil then
  begin
    Data := VSTCon.GetNodeData(Node);
    if Data <> nil then
      Data^.FCC := nil;
    VSTCon.DeleteNode(Node, False);
  end;
  if FClientStatus <> csEndSession then
    FClientStatus := csDisconnected;
  Setup(AConData);
end;

procedure TfMain.DoOnSocketError(Sender: TObject; AErrMsg: string; AErrCode: Integer;
 AConData: TConData);
begin
  AddLog(AErrMsg + '(' + IntToStr(AErrCode) + ').');
end;

procedure TfMain.DoOnInternalMessage(Sender: TObject; AMsg: String);
begin
  AddLog(AMsg);
end;

procedure TfMain.AddLog(ALogMsg: String);
begin
  lbLog.Items.Add(FormatDateTime('hh:mm:ss', now) + '  ' + ALogMsg);
  lbLog.TopIndex := lbLog.Items.Count - 1;
end;
{$EndRegion}

{$Region 'Menu Events'}
procedure TfMain.miClearClick(Sender: TObject);
begin
  lbLog.Clear;
end;

procedure TfMain.miOpenClick(Sender: TObject);
begin
  if FClientStatus = csEndSession then
    Exit;
  if fCommunication.Visible then
    fCommunication.BringToFront
  else
    fCommunication.Show;
end;

procedure TfMain.miShowErrorMessagesClick(Sender: TObject);
begin
  miShowErrorMessages.Checked := not miShowErrorMessages.Checked;
  FSettings.ShowErrorMessages := miShowErrorMessages.Checked;
  FSettings.Save;
end;

procedure TfMain.pmLogPopup(Sender: TObject);
begin
  FSettings.Load;
  miShowErrorMessages.Checked := FSettings.ShowErrorMessages;
end;

procedure TfMain.miSaveClick(Sender: TObject);
begin
  SD.FileName := 'Log';
  if SD.Execute then
    lbLog.Items.SaveToFile(SD.FileName);
end;

procedure TfMain.acQuitExecute(Sender: TObject);
begin
  Close;
end;

procedure TfMain.acOptionsExecute(Sender: TObject);
begin
  fOptions := TfOptions.Create(nil, FSettings);
  try
    fOptions.ShowModal;
    if fOptions.ModalResult = mrOk then
    begin
      FTCPClient.ShowErrorMessages := FSettings.ShowErrorMessages;
      FTCPClient.DownloadDirectory := FSettings.DownloadDirectory;
      lbLog.Visible := FSettings.ShowLog;
      sp.Visible := FSettings.ShowLog;
    end;
  finally
    fOptions.Free;
  end;
end;

procedure TfMain.acConnectExecute(Sender: TObject);
begin
  fLogin := TfLogin.Create(nil, FTCPClient, FSettings);
  try
    pnMain.Visible := False;
    fLogin.ShowModal;
    pnMain.Visible := True;
  finally
    fLogin.Free;
  end;
end;

procedure TfMain.acDisconnectExecute(Sender: TObject);
begin
  if FTCPClient.Connected then
  begin
    FByUser := True;
    Caption := 'Disconnecting. Please wait...';
    fMain.Enabled := False;
    Application.ProcessMessages;
    try
      FTCPClient.Disconnect;
    finally
      fMain.Enabled := True;
    end;
  end;
end;

procedure TfMain.acAboutExecute(Sender: TObject);
begin
  fAbout := TfAbout.Create(nil);
  try
    fAbout.ShowModal;
  finally
    fAbout.Free;
  end;
end;
{$EndRegion}

{$Region 'VirtualStringTree Events'}
procedure TfMain.VSTConCompareNodes(Sender: TBaseVirtualTree; Node1,
  Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
var
  Data1: PData;
  Data2: PData;
begin
  Data1 := Sender.GetNodeData(Node1);
  Data2 := Sender.GetNodeData(Node2);
  case Column of
    0: Result := CompareText(Data1^.FLocalUser, Data2^.FLocalUser);
    1: Result := CompareText(Data1^.FIP, Data2^.FIP);
    2: Result := CompareText(Data1^.FHost, Data2^.FHost);
  end;
end;

procedure TfMain.VSTConDblClick(Sender: TObject);
begin
  if FClientStatus = csEndSession then
    Exit;
  miOpenClick(miOpen);
end;

procedure TfMain.VSTConFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
var
  Data: PData;
begin
  Data := VSTCon.GetNodeData(Node);
  Finalize(Data^);
end;

procedure TfMain.VSTConGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
  Column: TColumnIndex; TextType: TVSTTextType; var CellText: String);
var
  Data: PData;
begin
  if TextType = ttStatic then
    Exit;
  Data := VSTCon.GetNodeData(Node);
  case Column of
    0: CellText := Data^.FLocalUser;
    1: CellText := Data^.FIP;
    2: CellText := Data^.FHost;
  end;
end;

procedure TfMain.VSTConPaintText(Sender: TBaseVirtualTree;
  const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType);
begin
  if Node = VSTCon.FocusedNode then
    TargetCanvas.Font.Color := clHighlightText
  else
    TargetCanvas.Font.Color := VSTCon.Font.Color;
end;

procedure TfMain.VSTConHeaderClick(Sender: TVTHeader;
  HitInfo: TVTHeaderHitInfo);
begin
  if (HitInfo.Column > 2) or (FClientStatus = csEndSession) then
    Exit;
  if HitInfo.Button = mbLeft then
  begin
    with Sender, Treeview do
    begin
      if (SortColumn = NoColumn) or (SortColumn <> HitInfo.Column) then
      begin
        SortColumn    := HitInfo.Column;
        SortDirection := laz.VirtualTrees.sdAscending;
      end
      else
      begin
        if SortDirection = laz.VirtualTrees.sdAscending then
          SortDirection := laz.VirtualTrees.sdDescending
        else
          SortDirection := laz.VirtualTrees.sdAscending;
        FSortDir := SortDirection;
      end;
      SortTree(SortColumn, SortDirection, False);
      FSortCol := Sender.SortColumn;
    end;
  end;
end;

procedure TfMain.VSTConMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  Node: PVirtualNode;
  P: TPoint;
begin
  if FClientStatus = csEndSession then
    Exit;
  if Button = mbRight then
  begin
    Node := VSTCon.GetNodeAt(X, Y);
    if (Node <> nil) and VSTCon.Selected[Node] then
    begin
      P.X := X; P.Y := Y;
      P := ClientToScreen(P);
      pmAction.PopUp(P.X, P.Y);
    end;
  end;
end;
{$EndRegion}


end.

