unit uOptions;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ComCtrls, StdCtrls,
  ExtCtrls, EditBtn, LazFileUtils, uSettings;

type

  { TfOptions }

  TfOptions = class(TForm)
    bCancel: TButton;
    bOk: TButton;
    bDefaults: TButton;
    cbAutoConnect: TCheckBox;
    cbConnection: TCheckBox;
    cbCredentials: TCheckBox;
    cbErrorMessages: TCheckBox;
    cbLog: TCheckBox;
    deDownloads: TDirectoryEdit;
    lbDownload: TLabel;
    pgOptions: TPageControl;
    pnBottom: TPanel;
    tsFiles: TTabSheet;
    tsMessages: TTabSheet;
    tsGeneral: TTabSheet;
    procedure bDefaultsClick(Sender: TObject);
    procedure bOkClick(Sender: TObject);
    procedure SettingsChanged(Sender: TObject);
    procedure cbConnectionClick(Sender: TObject);
    procedure cbLogClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: char);
  private
    FLoading: Boolean;
    FSettings: TSettings;
    procedure LoadSettings;
    procedure SaveSettings;
  public
    constructor Create(TheOwner: TComponent; ASettings: TSettings); reintroduce;
  end;

var
  fOptions: TfOptions;

implementation

{$R *.lfm}

{ TfOptions }

constructor TfOptions.Create(TheOwner: TComponent; ASettings: TSettings);
begin
  inherited Create(TheOwner);
  FSettings := ASettings;
  pgOptions.ActivePageIndex := 0;
  FLoading := True;
  LoadSettings;
  FLoading := False;
end;

procedure TfOptions.bDefaultsClick(Sender: TObject);
begin
  if MessageDlg('Restore default settings?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
  begin
    cbConnection.Checked := True;
    cbCredentials.Checked := True;
    cbAutoConnect.Checked := False;
    cbLog.Checked := True;
    cbErrorMessages.Checked := False;
    deDownloads.Directory := AppendPathDelim(GetUserDir) + 'Downloads';
  end;
end;

procedure TfOptions.bOkClick(Sender: TObject);
begin
  SaveSettings;
  ModalResult := mrOk;
end;

procedure TfOptions.cbConnectionClick(Sender: TObject);
begin
  cbErrorMessages.Enabled := cbLog.Checked;
  if not cbErrorMessages.Enabled then
    cbErrorMessages.Checked := False;
end;

procedure TfOptions.cbLogClick(Sender: TObject);
begin
  cbErrorMessages.Enabled := cbLog.Checked;
  if not cbErrorMessages.Enabled then
    cbErrorMessages.Checked := False;
end;

procedure TfOptions.FormKeyPress(Sender: TObject; var Key: char);
begin
  if Key = #27 then
    Close;
end;

procedure TfOptions.LoadSettings;
begin
  with FSettings do
  begin
    Load;
    cbConnection.Checked := SaveConnections;
    cbCredentials.Enabled := SaveConnections;
    cbCredentials.Checked := SaveCredentials;
    cbAutoConnect.Enabled := SaveConnections;
    cbAutoConnect.Checked := AutoConnect;
    cbLog.Checked := ShowLog;
    cbErrorMessages.Checked := ShowErrorMessages;
    cbErrorMessages.Enabled := cbLog.Checked;
    deDownloads.Directory := DownloadDirectory;
  end;
end;

procedure TfOptions.SaveSettings;
begin
  with FSettings do
  begin
    AutoConnect := cbAutoConnect.Checked;
    SaveConnections := cbConnection.Checked;
    SaveCredentials := cbCredentials.Checked;
    ShowLog := cbLog.Checked;
    ShowErrorMessages := cbErrorMessages.Checked;
    DownloadDirectory := deDownloads.Directory;
    Save;
  end;
end;

procedure TfOptions.SettingsChanged(Sender: TObject);
begin
  if FLoading then
    Exit;
  bOk.Enabled := True;
end;

end.

