unit uproxy;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls,
  Spin, uTCPSockets;

type
  TfProxy = class(TForm)
    bCancel: TButton;
    bvProxy: TBevel;
    bOk: TButton;
    cbProxyType: TComboBox;
    edProxyIP: TEdit;
    edProxyPass: TEdit;
    edProxyUser: TEdit;
    lbIP: TLabel;
    lbPass: TLabel;
    lbPort: TLabel;
    lbUser: TLabel;
    pnBottom: TPanel;
    spProxyPort: TSpinEdit;
    procedure bOkClick(Sender: TObject);
    procedure cbProxyTypeChange(Sender: TObject);
    procedure SettingsChanged(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: char);
    procedure FormShow(Sender: TObject);
  private
    FLoading: Boolean;
    FActiveConnection: TConnection;
    procedure EnableDisable(AEnable: Boolean);
    procedure GetValues;
    procedure SetValues;
  public
    constructor Create(TheOwner: TComponent; AActiveConnection: TConnection); reintroduce;
  end;

var
  fProxy: TfProxy;

implementation

{$R *.lfm}

constructor TfProxy.Create(TheOwner: TComponent; AActiveConnection: TConnection);
begin
  inherited Create(TheOwner);
  FActiveConnection := AActiveConnection;
end;

procedure TfProxy.FormShow(Sender: TObject);
begin
  FLoading := True;
  GetValues;
  FLoading := False;
end;

procedure TfProxy.FormKeyPress(Sender: TObject; var Key: char);
begin
  if Key = #27 then
    Close;
end;

procedure TfProxy.bOkClick(Sender: TObject);
begin
  if (cbProxyType.ItemIndex > 0) and (Trim(edProxyIP.Text) = '') then
  begin
    edProxyIp.SetFocus;
    MessageDlg('Please enter the proxy server.', mtInformation, [mbOk], 0);
    Exit;
  end;
  SetValues;
  ModalResult := mrOk;
end;

procedure TfProxy.cbProxyTypeChange(Sender: TObject);
begin
  SettingsChanged(Sender);
  EnableDisable(cbProxyType.ItemIndex <> 0);
  if cbProxyType.ItemIndex = 0 then
  begin
    edProxyIP.Text := '';
    spProxyPort.Value := DefProxyPort;
    edProxyUser.Text := '';
    edProxyPass.Text := '';
  end;
end;

procedure TfProxy.EnableDisable(AEnable: Boolean);
begin
  edProxyIP.Enabled := AEnable;
  lbIP.Enabled := AEnable;
  spProxyPort.Enabled := AEnable;
  lbPort.Enabled := AEnable;
  edProxyUser.Enabled := AEnable;
  lbUser.Enabled := AEnable;
  edProxyPass.Enabled := AEnable;
  lbPass.Enabled := AEnable;
end;

procedure TfProxy.GetValues;
var
  Port: Integer;
begin
  with FActiveConnection.FProxy do
  begin
    cbProxyType.ItemIndex := Ord(FProxyType);
    if cbProxyType.ItemIndex > 0 then
      EnableDisable(True);
    edProxyIP.Text := FProxyIP;
    Port := FProxyPort;
    if Port > 0 then
      spProxyPort.Value := Port;
    edProxyUser.Text := FProxyUser;
    edProxyPass.Text := FProxyPass;
  end;
end;

procedure TfProxy.SetValues;
begin
  with FActiveConnection.FProxy do
  begin
    FProxyType := TProxyType(cbProxyType.ItemIndex);
    if Ord(FProxyType) > 0 then
    begin
      FProxyIP   := edProxyIP.Text;
      FProxyPort := spProxyPort.Value;
      FProxyUser := edProxyUser.Text;
      FProxyPass := edProxyPass.Text;
    end
    else
    begin
      FProxyIP   := '';
      FProxyPort := DefProxyPort;
      FProxyUser := '';
      FProxyPass := '';
    end;
  end;
end;

procedure TfProxy.SettingsChanged(Sender: TObject);
begin
  if FLoading then
    Exit;
  bOk.Enabled := True;
end;

end.

