unit uFileTransfer;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, ExtCtrls, Graphics, Menus, Dialogs, StdCtrls,
  FileUtil, LCLIntf, LCLType, laz.VirtualTrees, uTCPSockets;

type
  PFileData = ^TFileData;
  TFileData = record
    FFileName: String;
    FProgress: Integer;
    FProgressStr: String;
    FWorkDirection: TWorkDirection;
    FSpeed: Int64;
    FSpeedStr: String;
    FRemaining: Int64;
    FRemainingStr: String;
    FSize: Int64;
    FSizeStr: String;
    FTotalSize: Int64;
    FTotalSizeStr: String;
    FWorker: TWorker;
  end;

  TOnChanged = procedure(Sender: TObject; AType: Byte) of object;
  TOnSendFiles = procedure(Sender: TObject; AFiles: TStrings) of object;
  TfrFileTransfer = class(TFrame)
    bSend: TButton;
    lbTitle: TLabel;
    miRemove: TMenuItem;
    OD: TOpenDialog;
    pmFiles: TPopupMenu;
    pnContainer: TPanel;
    pnControls: TPanel;
    pnTree: TPanel;
    sp: TSplitter;
    VSTFiles: TLazVirtualStringTree;
    procedure bSendClick(Sender: TObject);
    procedure miRemoveClick(Sender: TObject);
    procedure VSTFilesAfterCellPaint(Sender: TBaseVirtualTree;
      TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
      const CellRect: TRect);
    procedure VSTFilesCompareNodes(Sender: TBaseVirtualTree; Node1,
      Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
    procedure VSTFilesFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
    procedure VSTFilesGetHint(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; var LineBreakStyle: TVTTooltipLineBreakStyle;
      var HintText: String);
    procedure VSTFilesGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType; var CellText: String);
    procedure VSTFilesHeaderClick(Sender: TVTHeader; HitInfo: TVTHeaderHitInfo);
    procedure VSTFilesKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure VSTFilesMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure VSTFilesPaintText(Sender: TBaseVirtualTree;
      const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
      TextType: TVSTTextType);
  private
    FSortCol: Integer;
    FSortDir: laz.VirtualTrees.TSortDirection;
    FOnChanged: TOnChanged;
    FOnSendFiles: TOnSendFiles;
    procedure Cleanup;
  public
    procedure InitializeFrame(AType: Byte);
    procedure FinalizeFrame;
    procedure DoOnSentFile(Sender: TObject; AFileName: String; AConData: TConData);
    procedure DoOnRecvFile(Sender: TObject; AFileName: String; AConData: TConData);
    procedure DoOnFileProgress(Sender: TObject; AWorkDirection: TWorkDirection; AFileName: String; ACnt, ATotCnt, ASpeed, ARemaining: Int64; AConData: TConData);
  public
    property OnChanged: TOnChanged read FOnChanged write FOnChanged;
    property OnSendFiles: TOnSendFiles read FOnSendFiles write FOnSendFiles;
  end;

implementation

uses uFunctions;

{$R *.lfm}

procedure TfrFileTransfer.InitializeFrame(AType: Byte);
begin
  VSTFiles.NodeDataSize := SizeOf(TFileData);
  FSortCol := -1;
  case AType of
    0: begin
         pnContainer.Height := 50;
         pnTree.Width := 0;
         lbTitle.Caption := 'Send files to server';
         lbTitle.Visible := False;
       end;
    1: begin
         pnContainer.Height := 150;
         pnTree.Width := 175;
         lbTitle.Caption := 'Send files to selected users';
       end;
    2: begin
         pnContainer.Height := 150;
         pnTree.Width := 175;
         lbTitle.Caption := 'Send files from selected users to server';
       end;
  end;
  pnContainer.Constraints.MinHeight := pnContainer.Height;
end;

procedure TfrFileTransfer.FinalizeFrame;
begin
  //
end;

procedure TfrFileTransfer.DoOnSentFile(Sender: TObject; AFileName: String;
  AConData: TConData);
var
  Data: PFileData;
  Node: PVirtualNode;
begin
  Node := PVirtualNode(AConData.FData);
  if Node <> nil then
  begin
    Data := VSTFiles.GetNodeData(Node);
    Data^.FProgressStr := 'Completed';
    VSTFiles.ReinitNode(Node, False);
    VSTFiles.RepaintNode(Node);
  end;
end;

procedure TfrFileTransfer.DoOnRecvFile(Sender: TObject; AFileName: String;
  AConData: TConData);
var
  Data: PFileData;
  Node: PVirtualNode;
begin
  Node := PVirtualNode(AConData.FData);
  if Node <> nil then
  begin
    Data := VSTFiles.GetNodeData(Node);
    Data^.FProgressStr := 'Completed';
    VSTFiles.ReinitNode(Node, False);
    VSTFiles.RepaintNode(Node);
  end;
  if Assigned(FOnChanged) then
    FOnChanged(Self, 0);
end;

procedure TfrFileTransfer.DoOnFileProgress(Sender: TObject; AWorkDirection: TWorkDirection;
  AFileName: String; ACnt, ATotCnt, ASpeed, ARemaining: Int64; AConData: TConData);
var
  Node: PVirtualNode;
  Data: PFileData;
begin
  Node := PVirtualNode(AConData.FData);
  if Node = nil then
  begin
    Node := VSTFiles.AddChild(nil);
    AConData.FData := PVirtualNode(Node);
  end;

  Data := VSTFiles.GetNodeData(Node);
  Data^.FWorker := TWorker(Sender);
  Data^.FFileName := AFileName;
  Data^.FWorkDirection := AWorkDirection;

  if ACnt < ATotCnt then
    Data^.FProgress := Trunc(ACnt*100/ATotCnt)
  else
    Data^.FProgress := 100;
  case AWorkDirection of
    wdDownload: Data^.FProgressStr := 'Downloading ' + IntToStr(Data^.FProgress) + '%';
    wdUpload: Data^.FProgressStr := 'Uploading ' + IntToStr(Data^.FProgress) + '%';
  end;

  Data^.FSpeed := ASpeed;
  if ACnt < ATotCnt then
    Data^.FSpeedStr := FormatSpeed(ASpeed)
  else
    Data^.FSpeedStr := '';

  Data^.FRemaining := ARemaining;
  if ACnt < ATotCnt then
    Data^.FRemainingStr := SecToHourAndMin(ARemaining)
  else
    Data^.FRemainingStr := '';

  Data^.FSize := ACnt;
  if ACnt < ATotCnt then
    Data^.FSizeStr := FormatSize(ACnt)
  else
    Data^.FSizeStr := '';

  Data^.FTotalSize := ATotCnt;
  Data^.FTotalSizeStr := FormatSize(ATotCnt);
  if ACnt >= ATotCnt then
    Data^.FWorker := nil;

  if (ACnt = -1) and (ATotCnt = -1) then
  begin
    if Trim(Data^.FFileName) = '' then
      Data^.FFileName := 'unknown file';
    if Data^.FTotalSize = -1 then
      Data^.FTotalSizeStr := FormatSize(0);
    Data^.FWorker := nil;
    case Data^.FWorkDirection of
      wdDownload: Data^.FProgressStr := 'Error downloading file';
      wdUpload: Data^.FProgressStr := 'Error uploading file';
    end;
    Data^.FWorkDirection := wdError;
  end;

  VSTFiles.BeginUpdate;
  try
    VSTFiles.ReinitNode(Node, False);
    VSTFiles.RepaintNode(Node);
    if FSortCol <> -1 then
      VSTFiles.SortTree(FSortCol, FSortDir);
  finally
    VSTFiles.EndUpdate;
  end;
   if Assigned(FOnChanged) then
    FOnChanged(Self, 0);
end;

procedure TfrFileTransfer.VSTFilesAfterCellPaint(Sender: TBaseVirtualTree;
  TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  const CellRect: TRect);
const
  BackgroundStartColor = $00FCFCFC;
  BackgroundStopColor = $00C8C8C8;
  DownloadColor = $024A60D;
  UploadColor = $001C9EFF;
  ErrorColor = $005B5BFF;
var
  Data: PFileData;
  W: Integer;
  Str: String;
  Rect, DrawRect: TRect;
begin
  if Column = 1 then
  begin
    Data := VSTFiles.GetNodeData(Node);
    Rect.Create(CellRect);
    InflateRect(Rect, -2, -2);
    DrawRect.Create(Rect);
    with TargetCanvas do
    begin
      Brush.Color := clGray;
      FillRect(Rect.Left, Rect.Top, Rect.Right, Rect.Bottom);
      if (Node = Sender.FocusedNode) then
        Brush.Color := $00D7D7D7
      else
        Brush.Color := clWhite;
      FillRect(Rect.Left + 1, Rect.Top + 1, Rect.Right - 1, Rect.Bottom - 1);

      SetRect(DrawRect, Rect.Left + 2, Rect.Top + 2, Rect.Right - 2, Rect.Bottom - 2);
      if (Node = Sender.FocusedNode) then
        GradientFill(DrawRect, VSTFiles.Colors.SelectionRectangleBorderColor, VSTFiles.Colors.SelectionRectangleBorderColor, gdHorizontal)
      else
        GradientFill(DrawRect, BackgroundStartColor, BackgroundStopColor, gdHorizontal);

      if (Data^.FProgress > 0) then
      begin
        W := Trunc(Rect.Width*Data^.FProgress/100);
        if W > 2 then
        begin
          SetRect(DrawRect, Rect.Left + 2, Rect.Top + 2, Rect.Left + W - 2, Rect.Bottom - 2);
          case Data^.FWorkDirection of
            wdDownload: Brush.Color := DownloadColor;
            wdUpload: Brush.Color := UploadColor;
            wdError: Brush.Color := ErrorColor;
          end;
          FillRect(DrawRect);
        end;
      end;

      SetRect(DrawRect, Rect.Left + 2, Rect.Top + 2, Rect.Right - 2, Rect.Bottom - 2);
      SetBkMode(Handle, TRANSPARENT);
      Str := Data^.FProgressStr;
      if Node = VSTFiles.FocusedNode then
        Pen.Color := clHighlight
      else
        Pen.Color := VSTFiles.Font.Color;
      DrawText(Handle, PChar(Str), -1, Rect, DT_SINGLELINE or DT_CENTER or DT_VCENTER or DT_END_ELLIPSIS or DT_NOPREFIX);
    end;
  end;
end;

procedure TfrFileTransfer.VSTFilesCompareNodes(Sender: TBaseVirtualTree; Node1,
  Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
var
  Data1: PFileData;
  Data2: PFileData;
begin
  Data1 := Sender.GetNodeData(Node1);
  Data2 := Sender.GetNodeData(Node2);
  case Column of
    0: Result := CompareText(ExtractFileName(Data1^.FFileName), ExtractFileName(Data2^.FFileName));
    1: Result := Data1^.FProgress - Data2^.FProgress;
    2: Result := Data1^.FSpeed - Data2^.FSpeed;
    3: Result := Data1^.FRemaining - Data2^.FRemaining;
    4: Result := Data1^.FSize - Data2^.FSize;
    5: Result := Data1^.FTotalSize - Data2^.FTotalSize;
  end;
end;

procedure TfrFileTransfer.VSTFilesHeaderClick(Sender: TVTHeader;
  HitInfo: TVTHeaderHitInfo);
begin
  if HitInfo.Button = mbLeft then
  begin
    with Sender, Treeview do
    begin
      if (SortColumn = NoColumn) or (SortColumn <> HitInfo.Column) then
      begin
        SortColumn    := HitInfo.Column;
        SortDirection := laz.VirtualTrees.sdAscending;
      end
      else
      begin
        if SortDirection = laz.VirtualTrees.sdAscending then
          SortDirection := laz.VirtualTrees.sdDescending
        else
          SortDirection := laz.VirtualTrees.sdAscending;
        FSortDir := SortDirection;
      end;
      SortTree(SortColumn, SortDirection, False);
      FSortCol := Sender.SortColumn;
    end;
  end;
end;

procedure TfrFileTransfer.VSTFilesFreeNode(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
var
  Data: PFileData;
begin
  Data := VSTFiles.GetNodeData(Node);
  Finalize(Data^);
end;

procedure TfrFileTransfer.VSTFilesGetHint(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex;
  var LineBreakStyle: TVTTooltipLineBreakStyle; var HintText: String);
var
  Data: PFileData;
begin
  if Column <>  0 then
    Exit;
  Data := VSTFiles.GetNodeData(Node);
  LineBreakStyle := hlbForceSingleLine;
  HintText := Data^.FFileName;
end;

procedure TfrFileTransfer.VSTFilesGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: String);
var
  Data: PFileData;
begin
  if TextType = ttStatic then
    Exit;
  Data := VSTFiles.GetNodeData(Node);
  case Column of
    0: CellText := ExtractFileName(Data^.FFileName);
    1: CellText := '';
    2: CellText := Data^.FSpeedStr;
    3: CellText := Data^.FRemainingStr;
    4: CellText := Data^.FSizeStr;
    5: CellText := Data^.FTotalSizeStr;
  end;
end;

procedure TfrFileTransfer.VSTFilesKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (ssShift in Shift) and ((Key = VK_D) or (Key = VK_DELETE)) then
    miRemoveClick(miRemove);
end;

procedure TfrFileTransfer.VSTFilesMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Node: PVirtualNode;
  P: TPoint;
begin
  if Button = mbRight then
  begin
    Node := VSTFiles.GetNodeAt(X, Y);
    if (Node <> nil) and VSTFiles.Selected[Node] then
    begin
      P.X := X; P.Y := Y;
      P := ClientToScreen(P);
      pmFiles.PopUp(P.X, P.Y);
    end;
  end;
end;

procedure TfrFileTransfer.VSTFilesPaintText(Sender: TBaseVirtualTree;
  const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType);
begin
  if Node = VSTFiles.FocusedNode then
    TargetCanvas.Font.Color := clHighlightText
  else
    TargetCanvas.Font.Color := VSTFiles.Font.Color;
end;

procedure TfrFileTransfer.Cleanup;
var
  I: Integer;
  Len: Integer;
  Node: PVirtualNode;
  Nodes: array of PVirtualNode;
begin
  Nodes := nil;
  Node := VSTFiles.GetFirst;
  while Node <> nil do
  begin
    if not VSTFiles.IsVisible[Node] then
    begin
      Len := Length(Nodes);
      SetLength(Nodes, Len + 1);
      Nodes[Len] := Node;
    end;
    Node := VSTFiles.GetNext(Node);
  end;
  for I := High(Nodes) downto Low(Nodes) do
    VSTFiles.DeleteNode(Nodes[I]);
end;

procedure TfrFileTransfer.miRemoveClick(Sender: TObject);
var
  I: Integer;
  Len: Integer;
  Data: PFileData;
  Node: PVirtualNode;
  Nodes: array of PVirtualNode;
begin
  if MessageDlg('Remove selected?', mtConfirmation, [mbYes, mbNo], 0) <> mrYes then
    Exit;

  Nodes := nil;
  Node := VSTFiles.GetFirst;
  while Node <> nil do
  begin
    if VSTFiles.Selected[Node] then
    begin
      Len := Length(Nodes);
      SetLength(Nodes, Len + 1);
      Nodes[Len] := Node;
    end;
    Node := VSTFiles.GetNext(Node);
  end;
  for I := High(Nodes) downto Low(Nodes) do
  begin
    VSTFiles.IsVisible[Nodes[I]] := False;
    Data := VSTFiles.GetNodeData(Nodes[I]);
    if (Data^.FWorker <> nil) then
      Data^.FWorker.ForceQuit := True;
    VSTFiles.FocusedNode := VSTFiles.GetFirstSelected;
  end;
end;

procedure TfrFileTransfer.bSendClick(Sender: TObject);
begin
  Cleanup;
  if OD.Execute then
    if Assigned(FOnSendFiles) then
      FOnSendFiles(Self, OD.Files);
end;

end.

