unit uMessages;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, ExtCtrls, Graphics, Menus, Dialogs, StdCtrls,
  LCLIntf, LCLType, laz.VirtualTrees, uTCPSockets;

type
  TOnChanged = procedure(Sender: TObject; AType: Byte) of object;
  TOnSendMessage = procedure(Sender: TObject; AMsg: String) of object;
  TfrMessages = class(TFrame)
    bSend: TButton;
    edMsg: TEdit;
    lbTitle: TLabel;
    miClear: TMenuItem;
    pmMsg: TPopupMenu;
    pnContainer: TPanel;
    pnControls: TPanel;
    pnTree: TPanel;
    sp: TSplitter;
    VSTMsg: TLazVirtualStringTree;
    procedure bSendClick(Sender: TObject);
    procedure miClearClick(Sender: TObject);
    procedure VSTMsgFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
    procedure VSTMsgGetHint(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; var LineBreakStyle: TVTTooltipLineBreakStyle;
      var HintText: String);
    procedure VSTMsgGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType; var CellText: String);
    procedure VSTMsgKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure VSTMsgPaintText(Sender: TBaseVirtualTree;
      const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
      TextType: TVSTTextType);
  private
    FOnChanged: TOnChanged;
    FOnSendMessage: TOnSendMessage;
    procedure AddMessage(ATime: TTime; AFrom, ATo, AMsg: String);
  public
    procedure InitializeFrame(AType: Byte);
    procedure FinalizeFrame;
    procedure DoOnSentMessage(Sender: TObject; AMsg: String; AParams: TStringArray; AConData: TConData);
    procedure DoOnRecvMessage(Sender: TObject; AMsg: String; AParams: TStringArray; AConData: TConData);
  public
    property OnChanged: TOnChanged read FOnChanged write FOnChanged;
    property OnSendMessage: TOnSendMessage read FOnSendMessage write FOnSendMessage;
  end;

implementation

{$R *.lfm}

type
  PMsgData = ^TMsgData;
  TMsgData = record
    FTime: TTime;
    FFrom: String;
    FTo: String;
    FMsg: String;
  end;

procedure TfrMessages.InitializeFrame(AType: Byte);
begin
  VSTMsg.NodeDataSize := SizeOf(TMsgData);
  case AType of
    0: begin
         pnContainer.Height := 50;
         pnTree.Width := 0;
         lbTitle.Caption := 'Send message to server';
         lbTitle.Visible := False;
       end;
    1: begin
         pnContainer.Height := 150;
         pnTree.Width := 175;
         lbTitle.Caption := 'Send message to selected users';
       end;
    2: begin
         pnContainer.Height := 150;
         pnTree.Width := 175;
         lbtitle.Caption := 'Send message from selected users to server';
       end;
  end;
  pnContainer.Constraints.MinHeight := pnContainer.Height;
end;

procedure TfrMessages.FinalizeFrame;
begin
  //
end;

procedure TfrMessages.DoOnSentMessage(Sender: TObject; AMsg: String;
  AParams: TStringArray; AConData: TConData);
begin
  AddMessage(Now, AConData.FLocalUser, AConData.FRemoteUser, AMsg);
end;

procedure TfrMessages.DoOnRecvMessage(Sender: TObject; AMsg: String;
  AParams: TStringArray; AConData: TConData);
begin
  AddMessage(Now, AConData.FRemoteUser, AConData.FLocalUser, AMsg);
  if Assigned(FOnChanged) then
    FOnChanged(Self, 1);
end;

procedure TfrMessages.AddMessage(ATime: TTime; AFrom, ATo, AMsg: String);
var
  Data: PMsgData;
  Node: PVirtualNode;
begin
  VSTMsg.BeginUpdate;
  try
    Node := VSTMsg.AddChild(nil);
    Data := VSTMsg.GetNodeData(Node);
    Data^.FTime := ATime;
    Data^.FFrom := AFrom;
    Data^.FTo := ATo;
    Data^.FMsg := AMsg;
    VSTMsg.ScrollIntoView(Node, False);
  finally
    VSTMsg.EndUpdate;
  end;
end;

procedure TfrMessages.bSendClick(Sender: TObject);
begin
  if Trim(edMsg.Text) = '' then
  begin
    MessageDlg('Please enter a message!', mtInformation, [mbOk], 0);
    edMsg.SetFocus;
    Exit;
  end;
  if Assigned(FOnSendMessage) then
    FOnSendMessage(Self, edMsg.Text);
end;


procedure TfrMessages.miClearClick(Sender: TObject);
begin
  if MessageDlg('Clear messages?', mtConfirmation, [mbYes, mbNo], 0) <> mrYes then
    Exit;
  VSTMsg.BeginUpdate;
  try
    VSTMsg.Clear;
  finally
    VSTMsg.EndUpdate;
  end;
end;

procedure TfrMessages.VSTMsgFreeNode(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
var
  Data: PMsgData;
begin
  Data := VSTMsg.GetNodeData(Node);
  Finalize(Data^);
end;

procedure TfrMessages.VSTMsgGetHint(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex;
  var LineBreakStyle: TVTTooltipLineBreakStyle; var HintText: String);
var
  Data: PMsgData;
begin
  if Column <> 3 then
    Exit;
  Data := VSTMsg.GetNodeData(Node);
  LineBreakStyle := hlbForceSingleLine;
  HintText := Data^.FMsg;
end;

procedure TfrMessages.VSTMsgGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: String);
var
  Data: PMsgData;
begin
  Data := VSTMsg.GetNodeData(Node);
  case Column of
    0: CellText := FormatDateTime('hh:mm:ss', Data^.FTime);
    1: CellText := Data^.FFrom;
    2: CellText := Data^.FTo;
    3: CellText := Data^.FMsg;
  end;
end;

procedure TfrMessages.VSTMsgKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (ssShift in Shift) and ((Key = VK_D) or (Key = VK_DELETE)) then
    miClearClick(miClear);
end;

procedure TfrMessages.VSTMsgPaintText(Sender: TBaseVirtualTree;
  const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType);
begin
  if Node = VSTMsg.FocusedNode then
    TargetCanvas.Font.Color := clHighlightText
  else
    TargetCanvas.Font.Color := VSTMsg.Font.Color;
end;


end.

