program synapse_daemon;

{$mode objfpc}{$H+}

uses
  sysutils,
  classes,
  daemonapp,
  eventlog,
  uTCPSockets;

const
  Title = 'Synapse daemon';
  Version = '2.0';
  LogFile = 'synapse_daemon_' + Version + '.txt';

type
  TDaemon = class(TCustomDaemon)
  private
    FTCPServer: TTCPServer;
  public
    function ShutDown: Boolean; override;
    function Stop: Boolean; override;
    function Start: Boolean; override;
    function Pause: Boolean; override;
    function Continue: Boolean; override;
  end;

  TDaemonMapper = class(TCustomDaemonMapper)
  public
    constructor Create(AOwner: TComponent); override;
  end;

function TDaemon.Start: Boolean;
var
  Bind: TBind;
begin
  Result := inherited Start;
  Application.Log(etDebug, 'Daemon started.');

  Bind := TBind.Create;
  Bind.FIP := '0.0.0.0';
  Bind.FPort := DefPort;
  Bind.FUser := 'daemon';
  Bind.FPass := 'daemonpass';
  FTCPServer := TTCPServer.Create;
  FTCPServer.DownloadDirectory := 'c:\Synapse_Daemon\';
  FTCPServer.IsDaemon := True;
  FTCPServer.Start(Bind);
end;

function TDaemon.Stop: Boolean;
begin
  Result := inherited Stop;
  Application.Log(etDebug, 'Daemon stopped.');
end;

function TDaemon.Pause: Boolean;
begin
  Result := inherited Pause;
  Application.Log(etDebug, 'Daemon paused.');
end;

function TDaemon.Continue: Boolean;
begin
  Result := inherited Continue;
  Application.Log(etDebug, 'Daemon resumed.');
end;

function TDaemon.ShutDown: Boolean;
begin
  Result := inherited ShutDown;
  Application.Log(etDebug, 'Daemon shutdown.');
end;

constructor TDaemonMapper.Create(AOwner: TComponent);
var
  MyDaemonDef: TDaemonDef;
begin
  inherited Create(AOwner);

  Application.Log(etDebug, 'Daemon created.');
  MyDaemonDef := DaemonDefs.Add as TDaemonDef;
  MyDaemonDef.DaemonClassName := TDaemon.ClassName;
  MyDaemonDef.Name := 'SynapseDaemon';
  MyDaemonDef.DisplayName := 'SynapseDaemon';
  MyDaemonDef.Description := Title + ' ' + Version;
  MyDaemonDef.LogStatusReport := False;
  with MyDaemonDef.WinBindings do
  begin
    ServiceType := stWin32;
    StartType := stAuto;
    ErrorSeverity := esNormal;
    WaitHint := 0;
    IDTag := 0;
  end;
end;

begin
  RegisterDaemonClass(TDaemon);
  RegisterDaemonMapper(TDaemonMapper);
  with Application do
  begin
    Title := 'SynapseDaemonApplication';
    EventLog.LogType := ltFile;
    EventLog.DefaultEventType := etDebug;
    EventLog.AppendContent := true;
    EventLog.FileName := ExtractFilePath(Application.ExeName) + LogFile;
    Initialize;
    Run;
  end;
end.


