unit uCommunication;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ComCtrls, ExtCtrls,
  Menus, laz.VirtualTrees, uFileTransfer, uMessages, uTCPSockets, uMain;

type
  TfCommunication = class(TForm)
    pgCommunication: TPageControl;
    tsFiles: TTabSheet;
    tsMessages: TTabSheet;
    VSTClients: TLazVirtualStringTree;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure pgCommunicationChange(Sender: TObject);
    procedure VSTClientsAddToSelection(Sender: TBaseVirtualTree;
      Node: PVirtualNode);
    procedure VSTClientsBeforeCellPaint(Sender: TBaseVirtualTree;
      TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
      CellPaintMode: TVTCellPaintMode; CellRect: TRect; var ContentRect: TRect);
    procedure VSTClientsCompareNodes(Sender: TBaseVirtualTree; Node1,
      Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
    procedure VSTClientsFocusChanged(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex);
    procedure VSTClientsFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
    procedure VSTClientsGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType; var CellText: String);
    procedure VSTClientsHeaderClick(Sender: TVTHeader; HitInfo: TVTHeaderHitInfo);
    procedure VSTClientsPaintText(Sender: TBaseVirtualTree;
      const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
      TextType: TVSTTextType);
    procedure VSTClientsRemoveFromSelection(Sender: TBaseVirtualTree;
      Node: PVirtualNode);
  private
    FTCPBase: TTCPBase;
    FSortDir: laz.VirtualTrees.TSortDirection;
    FFileTransfer: TfrFileTransfer;
    FMessages: TfrMessages;
    procedure DoOnChanged(Sender: TObject; AType: Byte);
    procedure DoOnSendFiles(Sender: TObject; AFiles: TStrings);
    procedure DoOnSendMessage(Sender: TObject; AMsg: String);
  public
    constructor Create(TheOwner: TControl; ATCPBase: TTCPBase); reintroduce;
    procedure AddClientNode(ANode: PVirtualNode; AData: PData);
    procedure DeleteClientNode(ANode: PVirtualNode);
    procedure ChangeClientStatus(ANode: PVirtualNode; AClientStatus: TClientStatus);
  end;

var
  fCommunication: TfCommunication;

implementation

{$R *.lfm}

constructor TfCommunication.Create(TheOwner: TControl; ATCPBase: TTCPBase);
begin
  inherited Create(TheOwner);
  FTCPBase := ATCPBase;
end;

procedure TfCommunication.FormCreate(Sender: TObject);
begin
  Self.Width := fMain.Width + 150;
  Self.Height := fMain.Height - 20;
  VSTClients.Parent := Self;
  VSTClients.NodeDataSize := SizeOf(TData);

  FFileTransfer := TfrFileTransfer.Create(Self);
  FFileTransfer.Parent := tsFiles;
  FFileTransfer.Align := alClient;
  FFileTransfer.InitializeFrame(1);
  FFileTransfer.OnSendFiles := @DoOnSendFiles;
  FFileTransfer.OnChanged := @DoOnChanged;
  with fMain, FFileTransfer do
  begin
    FTCPBase.OnSentFile := @DoOnSentFile;
    FTCPBase.OnRecvFile := @DoOnRecvFile;
    FTCPBase.OnFileProgress := @DoOnFileProgress;
  end;

  FMessages := TfrMessages.Create(nil);
  FMessages.Parent := tsMessages;
  FMessages.Align := alClient;
  FMessages.InitializeFrame(1);
  FMessages.OnChanged := @DoOnChanged;
  FMessages.OnSendMessage := @DoOnSendMessage;
  with fMain, FMessages do
  begin
    FTCPBase.OnSentMessage := @DoOnSentMessage;
    FTCPBase.OnRecvMessage := @DoOnRecvMessage;
  end;

  pgCommunication.ActivePageIndex := 0;
  pgCommunicationChange(pgCommunication);
end;

procedure TfCommunication.FormDestroy(Sender: TObject);
begin
  FFileTransfer.FinalizeFrame;
  FFileTransfer.Free;
  FMessages.FinalizeFrame;
  FMessages.Free;
end;

procedure TfCommunication.pgCommunicationChange(Sender: TObject);
begin
  case pgCommunication.ActivePageIndex of
    0: VSTClients.Parent := FFileTransfer.pnTree;
    1: VSTClients.Parent := FMessages.pnTree;
  end;
end;

procedure TfCommunication.DoOnChanged(Sender: TObject; AType: Byte);
begin
  if not Self.Visible then
  begin
    Self.Visible := True;
    pgCommunication.ActivePageIndex := AType;
  end;
end;

procedure TfCommunication.DoOnSendFiles(Sender: TObject; AFiles: TStrings);
var
  I: Integer;
  Data: PData;
  Node: PVirtualNode;
begin
  if not TTCPServer(FTCPBase).Active then
  begin
    MessageDlg('Server is not active.', mtInformation, [mbOk], 0);
    Exit;
  end;

  for I := 0 to AFiles.Count - 1 do
  begin
    if not TTCPServer(FTCPBase).Active then
      Exit;
    Node := VSTClients.GetFirstSelected;
    while Node <> nil do
    begin
      if not TTCPServer(FTCPBase).Active then
        Exit;
      Data := VSTClients.GetNodeData(Node);
      if (Data <> nil) and (Data^.FAC <> nil) then
        TTCPServer(FTCPBase).SendFile(Data^.FAC, AFiles[I]);
      Node := VSTClients.GetNextSelected(Node);
    end;
  end;
end;

procedure TfCommunication.DoOnSendMessage(Sender: TObject; AMsg: String);
var
  Data: PData;
  Node: PVirtualNode;
begin
  if not TTCPServer(FTCPBase).Active then
  begin
    MessageDlg('Server is not active.', mtInformation, [mbOk], 0);
    Exit;
  end;

  Node := VSTClients.GetFirstSelected;
  while Node <> nil do
  begin
    if not TTCPServer(FTCPBase).Active then
      Break;
    Data := VSTClients.GetNodeData(Node);
    if (Data <> nil) and (Data^.FAC <> nil) then
      TTCPServer(FTCPBase).SendMessage(Data^.FAC, AMsg, []);
    Node := VSTClients.GetNextSelected(Node);
  end;
end;

{$Region 'Accepted Clients'}
procedure TfCommunication.AddClientNode(ANode: PVirtualNode; AData: PData);
var
  Data: PData;
  Node: PVirtualNode;
begin
  VSTClients.BeginUpdate;
  try
    Node := VSTClients.AddChild(nil);
    Data := VSTClients.GetNodeData(Node);
    Data^.FLocalUser := AData^.FLocalUser;
    Data^.FRemoteUser := AData^.FRemoteUser;
    Data^.FUniqueID := AData^.FUniqueID;
    Data^.FAC := AData^.FAC;
    Data^.FLinkNode := ANode;
    AData^.FLinkNode := Node;
    VSTClients.Sort(nil, 0, FSortDir);
  finally
    VSTClients.EndUpdate;
  end;
end;

procedure TfCommunication.DeleteClientNode(ANode: PVirtualNode);
var
  Data: PData;
  Node: PVirtualNode;
begin
  Data := VSTClients.GetNodeData(ANode);
  if (Data <> nil) and (Data^.FLinkNode <> nil) then
    Data^.FLinkNode := nil;
  VSTClients.DeleteNode(ANode);
  Node := VSTClients.GetFirstSelected;
  if Node <> nil then
    VSTClients.FocusedNode := Node;
end;

procedure TfCommunication.ChangeClientStatus(ANode: PVirtualNode;
  AClientStatus: TClientStatus);
var
  Data: PData;
begin
  Data := VSTClients.GetNodeData(ANode);
  Data^.FClientStatus := AClientStatus;
  case AClientStatus of
    csConnecting, csDisconnecting: Data^.FTick := GetTickCount64;
    csConnected, csDisconnected: Data^.FTick := 0;
  end;
  VSTClients.ReinitNode(ANode, False);
  VSTClients.RepaintNode(ANode);
end;

procedure TfCommunication.VSTClientsGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: String);
var
  Data: PData;
begin
  Data := Sender.GetNodeData(Node);
  case Column of
    0: CellText := Data^.FRemoteUser;
  end;
end;

procedure TfCommunication.VSTClientsCompareNodes(Sender: TBaseVirtualTree; Node1,
  Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
var
  Data1: PData;
  Data2: PData;
begin
  Data1 := Sender.GetNodeData(Node1);
  Data2 := Sender.GetNodeData(Node2);
  if Column = 0 then
    Result := CompareText(Data1^.FRemoteUser, Data2^.FRemoteUser);
end;

procedure TfCommunication.VSTClientsHeaderClick(Sender: TVTHeader;
  HitInfo: TVTHeaderHitInfo);
begin
  if (HitInfo.Column > 0) then
    Exit;
  if HitInfo.Button = mbLeft then
  begin
    with Sender, Treeview do
    begin
      if SortDirection = laz.VirtualTrees.sdAscending then
        SortDirection := laz.VirtualTrees.sdDescending
      else
        SortDirection := laz.VirtualTrees.sdAscending;
      FSortDir := SortDirection;
      Sort(nil, 0, SortDirection);
    end;
  end;
end;

procedure TfCommunication.VSTClientsBeforeCellPaint(Sender: TBaseVirtualTree;
  TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  CellPaintMode: TVTCellPaintMode; CellRect: TRect; var ContentRect: TRect);
var
  Data: PData;
begin
  if CellPaintMode = cpmPaint then
  begin
    Data := Sender.GetNodeData(Node);
    if (Node <> Sender.FocusedNode) then
    begin
      if fMain.Settings.HighlightConnection then
      begin
        case Data^.FClientStatus of
          csConnecting: TargetCanvas.Brush.Color := fMain.Settings.HighlightConnected;
          csConnected: TargetCanvas.Brush.Color := clWindow;
          csDisconnecting: TargetCanvas.Brush.Color := fMain.Settings.HighlightDisconnected;
          csDisconnected: TargetCanvas.Brush.Color := clWindow;
        end;
      end
      else
        TargetCanvas.Brush.Color := clWindow;
      TargetCanvas.FillRect(CellRect);
    end
    else
    begin
      TargetCanvas.Brush.Color := clHighlight;
      TargetCanvas.FillRect(CellRect);
    end;
  end;
end;

procedure TfCommunication.VSTClientsPaintText(Sender: TBaseVirtualTree;
  const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType);
begin
  if Node = Sender.FocusedNode then
    TargetCanvas.Font.Color := clHighlightText
  else
    TargetCanvas.Font.Color := Sender.Font.Color;
end;

procedure TfCommunication.VSTClientsFocusChanged(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex);
var
  Data: PData;
begin
  Data := VSTClients.GetNodeData(Node);
  if (Data <> nil) and (Data^.FLinkNode <> nil) then
    fMain.VSTCon.FocusedNode := Data^.FLinkNode;
end;

procedure TfCommunication.VSTClientsAddToSelection(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
var
  Data: PData;
begin
  Data := VSTClients.GetNodeData(Node);
  if (Data <> nil) and (Data^.FLinkNode <> nil) then
    fMain.VSTCon.Selected[Data^.FLinkNode] := True;
end;

procedure TfCommunication.VSTClientsRemoveFromSelection(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
var
  Data: PData;
begin
  Data := VSTClients.GetNodeData(Node);
  if (Data <> nil) and (Data^.FLinkNode <> nil) then
    fMain.VSTCon.Selected[Data^.FLinkNode] := False;
end;

procedure TfCommunication.VSTClientsFreeNode(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
var
  Data: PData;
begin
  Data := VSTClients.GetNodeData(Node);
  Finalize(Data^);
end;
{$EndRegion}


end.

