unit uMain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ComCtrls,
  ExtCtrls, LazFileUtils, laz.VirtualTrees, Menus, Buttons, ActnList,
  LCLType, LCLIntf, uSettings, uTCPSockets;

const
  Title = 'Synapse server';
  Version = '2.0';
  SynTrunkRev = 267;
  ConfigFile = 'synapse_server_' + Version + '.xml';

type
  TClientStatus = (csConnecting, csConnected, csDisconnecting, csDisconnected);
  PData = ^TData;
  TData = record
    FIP: String;
    FHost: String;
    FLocalUser: String;
    FRemoteUser: String;
    FUniqueID: Integer;
    FClientStatus: TClientStatus;
    FTick: QWord;
    FLinkNode: PVirtualNode;
    FAC: TAcceptedClient;
  end;

  TWaitType = (wtStart, wtClose);
  TServerStatus = (ssBeginSession, ssStarted, ssStopped, ssEndSession);
  TfMain = class(TForm)
    acQuit: TAction;
    acAbout: TAction;
    acStop: TAction;
    acStart: TAction;
    alMain: TActionList;
    acOptions: TAction;
    imSB: TImageList;
    lbLog: TListBox;
    MenuItem1: TMenuItem;
    miClear: TMenuItem;
    miDisconnect: TMenuItem;
    miShow: TMenuItem;
    miShowErrorMessages: TMenuItem;
    miOpen: TMenuItem;
    miSave: TMenuItem;
    mm: TMainMenu;
    miAbout: TMenuItem;
    miHelp: TMenuItem;
    miStop: TMenuItem;
    miStart: TMenuItem;
    miServer: TMenuItem;
    miFile: TMenuItem;
    miTools: TMenuItem;
    miOptions: TMenuItem;
    miQuit: TMenuItem;
    OD: TOpenDialog;
    pmAction: TPopupMenu;
    pmLog: TPopupMenu;
    pnMain: TPanel;
    sb: TStatusBar;
    SD: TSaveDialog;
    sp: TSplitter;
    tmTree: TTimer;
    tmWait: TTimer;
    VSTCon: TLazVirtualStringTree;
    procedure acQuitExecute(Sender: TObject);
    procedure acOptionsExecute(Sender: TObject);
    procedure acAboutExecute(Sender: TObject);
    procedure acStartExecute(Sender: TObject);
    procedure acStopExecute(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var {%H-}CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure miClearClick(Sender: TObject);
    procedure miDisconnectClick(Sender: TObject);
    procedure miOpenClick(Sender: TObject);
    procedure miShowErrorMessagesClick(Sender: TObject);
    procedure miSaveClick(Sender: TObject);
    procedure pmLogPopup(Sender: TObject);
    procedure sbDrawPanel(StatusBar: TStatusBar; Panel: TStatusPanel;
      const Rect: TRect);
    procedure tmTreeTimer(Sender: TObject);
    procedure tmWaitTimer(Sender: TObject);
    procedure VSTConAddToSelection(Sender: TBaseVirtualTree; Node: PVirtualNode);
    procedure VSTConBeforeCellPaint(Sender: TBaseVirtualTree;
      TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
      CellPaintMode: TVTCellPaintMode; CellRect: TRect; var ContentRect: TRect);
    procedure VSTConCompareNodes(Sender: TBaseVirtualTree; Node1,
      Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
    procedure VSTConDblClick(Sender: TObject);
    procedure VSTConFocusChanged(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex);
    procedure VSTConFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
    procedure VSTConGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType; var CellText: String);
    procedure VSTConHeaderClick(Sender: TVTHeader; HitInfo: TVTHeaderHitInfo);
    procedure VSTConMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure VSTConPaintText(Sender: TBaseVirtualTree;
      const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
      TextType: TVSTTextType);
    procedure VSTConRemoveFromSelection(Sender: TBaseVirtualTree;
      Node: PVirtualNode);
  private
    FSortCol: Integer;
    FSortDir: laz.VirtualTrees.TSortDirection;
    FBusy: Boolean;
    FServerStatus: TServerStatus;
    FTCPServer: TTCPServer;
    FSettings: TSettings;
    procedure Setup;
    procedure AddLog(ALogMsg: String);
    procedure ChangeStatus(ANode: PVirtualNode; AClientStatus: TClientStatus);
    procedure DoOnConnected(Sender: TObject; AConData: TConData; AConCnt: Integer);
    procedure DoOnDisconnected(Sender: TObject; AConData: TConData; AConCnt: Integer; AByUser: Boolean);
    procedure DoOnSocketError(Sender: TObject; AErrMsg: string; AErrCode: Integer; AConData: TConData);
    procedure DoOnInternalMessage(Sender: TObject; AMsg: String);
  public
    property Settings: TSettings read FSettings;
  end;

var
  fMain: TfMain;

implementation

uses uCommunication, uOptions, uStart, uAbout;

{$R *.lfm}

procedure TfMain.FormCreate(Sender: TObject);
var
  Path: String;
begin
  FSettings := TSettings.Create;
  Path := GetAppConfigDir(False);
  if not DirectoryExists(Path) then
    if not ForceDirectory(Path) then
      Path := GetTempDir(True);
  FSettings.FileName := Path  + ConfigFile;
  FSettings.Load;

  FTCPServer := TTCPServer.Create;
  FTCPServer.OnConnected := @DoOnConnected;
  FTCPServer.OnDisconnected := @DoOnDisconnected;
  FTCPServer.OnSocketError := @DoOnSocketError;
  fCommunication := TfCommunication.Create(Self, FTCPServer);
  Self.Width := 720;
  Self.Height := 450;
  Application.Title := Title + ' ' + Version;
  VSTCon.NodeDataSize := SizeOf(TData);
  FSortCol := 0;
  FSortDir := laz.VirtualTrees.sdAscending;
end;

procedure TfMain.FormShow(Sender: TObject);
begin
  FServerStatus := ssBeginSession;
  Setup;
end;

procedure TfMain.tmWaitTimer(Sender: TObject);
begin
  case TWaitType(tmWait.Tag) of
    wtStart:
      begin
        tmWait.Enabled := False;
        acStartExecute(acStart);
      end;
    wtClose:
      if not FTCPServer.Active then
      begin
        tmWait.Enabled := False;
        Application.Terminate;
      end
      else
      begin
        FTCPServer.Stop;
        VSTCon.Clear;
        fCommunication.VSTClients.Clear;
      end;
  end;
end;

procedure TfMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := MessageDlg('Are you sure you wish to quit?', mtConfirmation, [mbYes, mbNo], 0) = mrYes;
  if not CanClose then
    Exit;
  if FTCPServer.Active then
  begin
    CanClose := False;
    FServerStatus := ssEndSession;
    Setup;
  end;
end;

procedure TfMain.FormDestroy(Sender: TObject);
begin
  FTCPServer.Free;
  FSettings.Free;
end;

procedure TfMain.FormResize(Sender: TObject);
begin
  VSTCon.Repaint;
  lbLog.Repaint;
end;

procedure TfMain.sbDrawPanel(StatusBar: TStatusBar; Panel: TStatusPanel;
  const Rect: TRect);
var
  H, T, L: Integer;
begin
  with StatusBar.Canvas do
  begin
    H := StatusBar.Canvas.GetTextHeight('gettextheight');
    T := Rect.Top + (Rect.Bottom - Rect.Top - H) div 2;
    L := Rect.Left + 5;
    case Panel.Index of
      0: begin
          case sb.Tag of
             0: imSB.Draw(StatusBar.Canvas, L, T, 0);
             1: imSB.Draw(StatusBar.Canvas, L, T, 1);
             2: imSB.Draw(StatusBar.Canvas, L, T, 2);
           end;
           TextRect(Rect, L + ImSb.Width + 5, T, Panel.Text);
         end;
      1: TextRect(Rect, L + 5 , T, Panel.Text);
    end;
  end;
end;

procedure TfMain.tmTreeTimer(Sender: TObject);
var
  I: Integer;
  Len: Integer;
  Data: PData;
  Node: PVirtualNode;
  Nodes: array of PVirtualNode;
  Tick: QWord;
begin
  if (FBusy) or (FServerStatus = ssEndSession) then
    Exit;
  FBusy := True;
  try
    Nodes := nil;
    Node := VSTCon.GetFirst;
    while Node <> nil do
    begin
      Data := VSTCon.GetNodeData(Node);
      if (Data^.FTick > 0) then
      begin
        Tick := GetTickCount64;
        if (Tick - Data^.FTick) > 1000*FSettings.HighlightInterval then
        case Data^.FClientStatus of
          csConnecting: ChangeStatus(Node, csConnected);
          csDisconnecting: ChangeStatus(Node, csDisconnected);
        end;
      end;
      if Data^.FClientStatus = csDisconnected then
      begin
        Len := Length(Nodes);
        SetLength(Nodes, Len + 1);
        Nodes[Len] := Node;
      end;
      Node := VSTCon.GetNext(Node);
    end;
    for I := High(Nodes) downto Low(Nodes) do
    begin
      Data := VSTCon.GetNodeData(Nodes[I]);
      fCommunication.DeleteClientNode(Data^.FLinkNode);
      VSTCon.DeleteNode(Nodes[I]);
      Node := VSTCon.GetFirstSelected;
      if Node <> nil then
        VSTCon.FocusedNode := Node;
    end;
  finally
    FBusy := False;
  end;
end;

procedure TfMain.Setup;
var
  LogStr: String;
begin
  case FServerStatus of
    ssBeginSession:
      begin
        Caption := Title + ' - inactive';
        LogStr := 'Idle. Waiting for commands';
        AddLog(LogStr);
        sb.Tag := 0;
        sb.Panels[0].Text := LogStr;
        sb.Panels[1].Text := 'Connections: 0';
        sp.Visible := FSettings.ShowLog;
        acStart.Enabled := True;
        acStop.Enabled := False;
        lbLog.Visible := FSettings.ShowLog;
        tmWait.Tag := Ord(wtStart);
        tmWait.Enabled := True;
      end;
    ssStarted:
      begin
        Caption := Title + ' - active';
        if Trim(FSettings.Bind.FPass) <> '' then
          LogStr := 'Server successfully started, listening on port ' + FSettings.Bind.FPort.ToString + '. Password protection is on.'
        else
          LogStr := 'Server successfully started, listening on port ' + FSettings.Bind.FPort.ToString + '. Password protection is off.';
        AddLog(LogStr);
        sb.Tag := 1;
        sb.Panels[0].Text := 'Server started';
        acStart.Enabled := False;
        acStop.Enabled := True;
        tmTree.Enabled := True;
      end;
    ssStopped:
      begin
        Caption := Title + ' - inactive';
        LogStr := 'Server stopped';
        AddLog(LogStr);
        sb.Tag := 2;
        sb.Panels[0].Text := LogStr;
        sb.Panels[1].Text := 'Connections: 0';
        acStart.Enabled := True;
        acStop.Enabled := False;
        tmTree.Enabled := False;
      end;
    ssEndSession:
      begin
        Caption := 'Stopping server. Please wait...';
        fMain.Enabled := False;
        FTCPServer.OnConnected := nil;
        FTCPServer.OnDisconnected := nil;
        FTCPServer.OnSocketError := nil;
        fCommunication.Close;
        tmWait.Tag := Ord(wtClose);
        tmWait.Enabled := True;
      end;
   end;
   Application.ProcessMessages;
end;

{$Region 'TCPClient events'}
procedure TfMain.DoOnConnected(Sender: TObject; AConData: TConData;
  AConCnt: Integer);
var
  Node: PVirtualNode;
  Data: PData;
begin
  VSTCon.BeginUpdate;
  try
    Node := VSTCon.AddChild(nil);
    Data := VSTCon.GetNodeData(Node);
    Data^.FIP := AConData.FRemoteIP;
    Data^.FHost := AConData.FRemoteHost;
    Data^.FLocalUser := AConData.FLocalUser;
    Data^.FRemoteUser := AConData.FRemoteUser;
    Data^.FUniqueID := AConData.FUniqueID;
    Data^.FAC := TAcceptedClient(Sender);
    if FSettings.HighlightConnection then
      ChangeStatus(Node, csConnecting)
    else
      ChangeStatus(Node, csConnected);
    AConData.FData := Node;
    VSTCon.Sort(nil, FSortCol, FSortDir);
    fCommunication.AddClientNode(Node, Data);
    if VSTCon.FocusedNode = nil then
    begin
      VSTCon.FocusedNode := Node;
      VSTCon.Selected[Node] := True;
    end;
  finally
    VSTCon.EndUpdate;
  end;
  sb.Panels[1].Text := 'Connections: ' + AConCnt.ToString;
  AddLog('Connection accepted from: ' + AConData.FRemoteIP + '. User "' + AConData.FRemoteUser + '" has successfully logged in.');
end;

procedure TfMain.DoOnDisconnected(Sender: TObject; AConData: TConData;
  AConCnt: Integer; AByUser: Boolean);
var
  Node: PVirtualNode;
  Data: PData;
begin
  VSTCon.BeginUpdate;
  try
    Node := PVirtualNode(AConData.FData);
    if Node <> nil then
    begin
      Data := VSTCon.GetNodeData(Node);
      if FSettings.HighlightConnection then
        ChangeStatus(Node, csDisconnecting)
      else
        ChangeStatus(Node, csDisconnected);
      if Data <> nil then
        Data^.FAC := nil;
    end;
  finally
    VSTCon.EndUpdate;
  end;
  sb.Panels[1].Text := 'Connections: ' + AConCnt.ToString;
  if AByUser then
    AddLog(AConData.FRemoteIP + ' disconnected by user.')
  else
    AddLog(AConData.FRemoteIP + ' disconnected. User "'  +  AConData.FRemoteUser + '" has logged out.');
end;

procedure TfMain.DoOnSocketError(Sender: TObject; AErrMsg: string; AErrCode: Integer;
  AConData: TConData);
begin
  if (AConData = nil) or ((AConData <> nil) and (AConData.FUniqueID = 0)) then
    AddLog('Sever error. ' + AErrMsg + '(' + IntToStr(AErrCode) + ')')
  else
    AddLog('Error for ' + AConData.FRemoteIP + '. ' + AErrMsg + '(' + IntToStr(AErrCode) + ')');
end;

procedure TfMain.DoOnInternalMessage(Sender: TObject; AMsg: String);
begin
  AddLog(AMsg);
end;

procedure TfMain.AddLog(ALogMsg: String);
begin
  lbLog.Items.BeginUpdate;
  lbLog.Items.Add(FormatDateTime('hh:mm:ss', now) + '  ' + ALogMsg);
  lbLog.TopIndex := lbLog.Items.Count - 1;
  lbLog.Items.EndUpdate;
  Application.ProcessMessages;
end;
{$EndRegion}

{$Region 'Menu Events'}
procedure TfMain.acQuitExecute(Sender: TObject);
begin
  Close;
end;

procedure TfMain.acStartExecute(Sender: TObject);
begin
  fStart := TfStart.Create(nil, FTCPServer, FSettings);
  try
    pnMain.Visible := False;
    fStart.ShowModal;
    if fStart.ModalResult = mrOk then
    begin
      FServerStatus := ssStarted;
      Setup;
    end;
    pnMain.Visible := True;
  finally
    fStart.Free;
  end;
end;

procedure TfMain.acStopExecute(Sender: TObject);
begin
  if FTCPServer.Active then
  begin
    Caption := 'Stopping server. Please wait...';
    fMain.Enabled := False;
    Application.ProcessMessages;
    try
      FTCPServer.Stop;
      VSTCon.Clear;
      fCommunication.VSTClients.Clear;
    finally
      fMain.Enabled := True;
    end;
  end;
  FServerStatus := ssStopped;
  Setup;
end;

procedure TfMain.acOptionsExecute(Sender: TObject);
begin
  fOptions := TfOptions.Create(nil, FSettings, FTCPServer.Active);
  try
    fOptions.ShowModal;
    lbLog.Visible := FSettings.ShowLog;
    sp.Visible := FSettings.ShowLog;
    FTCPServer.ShowErrorMessages := FSettings.ShowErrorMessages;
    FTCPServer.DownloadDirectory := FSettings.DownloadDirectory;
    VSTCon.Repaint;
  finally
    fOptions.Free;
  end;
end;

procedure TfMain.acAboutExecute(Sender: TObject);
begin
  fAbout := TfAbout.Create(nil);
  try
    fAbout.ShowModal;
  finally
    fAbout.Free;
  end;
end;

procedure TfMain.miClearClick(Sender: TObject);
begin
  lbLog.Clear;
end;

procedure TfMain.miDisconnectClick(Sender: TObject);
var
  I: Integer;
  Len: Integer;
  Data: PData;
  Node: PVirtualNode;
  ACTArray: array of TAcceptedClient;
begin
  ACTArray := nil;
  Node := VSTCon.GetFirstSelected;
  while Node <> nil do
  begin
    Data := VSTCon.GetNodeData(Node);
    if Data^.FAC <> nil then
    begin
      Len := Length(ACTArray);
      SetLength(ACTArray, Len + 1);
      ACTArray[Len] := Data^.FAC;
    end;
    Node := VSTCon.GetNextSelected(Node);
  end;

  for I := Low(ACTArray) to High(ACTArray) do
    if ACTArray[I] <> nil then
      TAcceptedClient(ACTArray[I]).ForceQuit := True;
 end;

procedure TfMain.miOpenClick(Sender: TObject);
begin
  if FServerStatus = ssEndSession then
    Exit;
  if fCommunication.Visible then
    fCommunication.BringToFront
  else
    fCommunication.Show;
end;

procedure TfMain.miSaveClick(Sender: TObject);
begin
  SD.FileName := 'Log';
  if SD.Execute then
    lbLog.Items.SaveToFile(SD.FileName);
end;

procedure TfMain.miShowErrorMessagesClick(Sender: TObject);
begin
  miShowErrorMessages.Checked := not miShowErrorMessages.Checked;
  FSettings.ShowErrorMessages := miShowErrorMessages.Checked;
  FSettings.Save;
end;

procedure TfMain.pmLogPopup(Sender: TObject);
begin
  FSettings.Load;
  miShowErrorMessages.Checked := FSettings.ShowErrorMessages;
end;
{$EndRegion}

{$Region 'VirtualStringTree Events'}
procedure TfMain.ChangeStatus(ANode: PVirtualNode; AClientStatus: TClientStatus);
var
  Data: PData;
begin
  Data := VSTCon.GetNodeData(ANode);
  Data^.FClientStatus := AClientStatus;
  case AClientStatus of
    csConnecting, csDisconnecting: Data^.FTick := GetTickCount64;
    csConnected, csDisconnected: Data^.FTick := 0;
  end;
  VSTCon.ReinitNode(ANode, False);
  VSTCon.RepaintNode(ANode);
  if Data^.FLinkNode <> nil then
    fCommunication.ChangeClientStatus(Data^.FLinkNode, AClientStatus);
end;

procedure TfMain.VSTConCompareNodes(Sender: TBaseVirtualTree; Node1,
  Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
var
  Data1: PData;
  Data2: PData;
begin
  Data1 := Sender.GetNodeData(Node1);
  Data2 := Sender.GetNodeData(Node2);
  case Column of
    0: Result := CompareText(Data1^.FRemoteUser, Data2^.FRemoteUser);
    1: Result := CompareText(Data1^.FIP, Data2^.FIP);
    2: Result := CompareText(Data1^.FHost, Data2^.FHost);
  end;
end;

procedure TfMain.VSTConGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
  Column: TColumnIndex; TextType: TVSTTextType; var CellText: String);
var
  Data: PData;
begin
  if TextType = ttStatic then
    Exit;
  Data := VSTCon.GetNodeData(Node);
  case Column of
    0: CellText := Data^.FRemoteUser;
    1: CellText := Data^.FIP;
    2: CellText := Data^.FHost;
    3: CellText := IntToStr(Data^.FUniqueID);
  end;
end;

procedure TfMain.VSTConPaintText(Sender: TBaseVirtualTree;
  const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType);
begin
  if Node = VSTCon.FocusedNode then
    TargetCanvas.Font.Color := clHighlightText
  else
    TargetCanvas.Font.Color := VSTCon.Font.Color;
end;

procedure TfMain.VSTConHeaderClick(Sender: TVTHeader;
  HitInfo: TVTHeaderHitInfo);
begin
  if (HitInfo.Column > 2) or (FServerStatus = ssEndSession) then
    Exit;
  if HitInfo.Button = mbLeft then
  begin
    with Sender, Treeview do
    begin
      if (SortColumn = NoColumn) or (SortColumn <> HitInfo.Column) then
      begin
        SortColumn    := HitInfo.Column;
        SortDirection := laz.VirtualTrees.sdAscending;
      end
      else
      begin
        if SortDirection = laz.VirtualTrees.sdAscending then
          SortDirection := laz.VirtualTrees.sdDescending
        else
          SortDirection := laz.VirtualTrees.sdAscending;
        FSortDir := SortDirection;
      end;
      SortTree(SortColumn, SortDirection, False);
      FSortCol := Sender.SortColumn;
    end;
  end;
end;

procedure TfMain.VSTConBeforeCellPaint(Sender: TBaseVirtualTree;
  TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  CellPaintMode: TVTCellPaintMode; CellRect: TRect; var ContentRect: TRect);
var
  Data: PData;
begin
  if CellPaintMode = cpmPaint then
  begin
    Data := Sender.GetNodeData(Node);
    if (Node <> Sender.FocusedNode) then
    begin
      if FSettings.HighlightConnection then
      begin
        case Data^.FClientStatus of
          csConnecting: TargetCanvas.Brush.Color := FSettings.HighlightConnected;
          csConnected: TargetCanvas.Brush.Color := clWindow;
          csDisconnecting: TargetCanvas.Brush.Color := FSettings.HighlightDisconnected;
          csDisconnected: TargetCanvas.Brush.Color := clWindow;
        end;
      end
      else
        TargetCanvas.Brush.Color := clWindow;
      TargetCanvas.FillRect(CellRect);
    end
    else
    begin
      TargetCanvas.Brush.Color := clHighlight;
      TargetCanvas.FillRect(CellRect);
    end;
  end;
end;

procedure TfMain.VSTConAddToSelection(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
var
  Data: PData;
begin
  if FServerStatus = ssEndSession then
    Exit;
  Data := VSTCon.GetNodeData(Node);
  if (Data <> nil) and (Data^.FLinkNode <> nil) then
    fCommunication.VSTClients.Selected[Data^.FLinkNode] := True;
end;

procedure TfMain.VSTConRemoveFromSelection(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
var
  Data: PData;
begin
  if FServerStatus = ssEndSession then
    Exit;
  Data := VSTCon.GetNodeData(Node);
  if (Data <> nil) and (Data^.FLinkNode <> nil) then
    fCommunication.VSTClients.Selected[Data^.FLinkNode] := False;
end;

procedure TfMain.VSTConFocusChanged(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex);
var
  Data: PData;
begin
  if FServerStatus = ssEndSession then
    Exit;
  Data := VSTCon.GetNodeData(Node);
  if (Data <> nil) and (Data^.FLinkNode <> nil) then
    fCommunication.VSTClients.FocusedNode:= Data^.FLinkNode;
end;

procedure TfMain.VSTConFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
var
  Data: PData;
begin
  Data := VSTCon.GetNodeData(Node);
  Finalize(Data^);
end;

procedure TfMain.VSTConDblClick(Sender: TObject);
begin
  if FServerStatus = ssEndSession then
    Exit;
  miOpenClick(miOpen);
end;

procedure TfMain.VSTConMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  Node: PVirtualNode;
  P: TPoint;
begin
  if FServerStatus = ssEndSession then
    Exit;
  if Button = mbRight then
  begin
    Node := VSTCon.GetNodeAt(X, Y);
    if (Node <> nil) and VSTCon.Selected[Node] then
    begin
      P.X := X; P.Y := Y;
      P := ClientToScreen(P);
      pmAction.PopUp(P.X, P.Y);
    end;
  end;
end;
{$EndRegion}

end.

