unit uOptions;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ComCtrls, StdCtrls,
  ExtCtrls, EditBtn, Spin, LazFileUtils, uSettings;

type
  TfOptions = class(TForm)
    bCancel: TButton;
    bOk: TButton;
    bDefaults: TButton;
    cbAutoStart: TCheckBox;
    cbErrorMessages: TCheckBox;
    cbCredentials: TCheckBox;
    cbHighlightConnection: TCheckBox;
    cbLog: TCheckBox;
    CD: TColorDialog;
    deDownloads: TDirectoryEdit;
    edServerName: TEdit;
    lbServerName: TLabel;
    lbHighlightConnected: TLabel;
    lbHighlightDisconnected: TLabel;
    lbDownload: TLabel;
    lbSec: TLabel;
    pgOptions: TPageControl;
    pnBottom: TPanel;
    pnHighlightConnected: TPanel;
    pnHighlightDisconnected: TPanel;
    pnHighlight: TPanel;
    shHighlightConnected: TShape;
    shHighlightDisconnected: TShape;
    spHighlightInterval: TSpinEdit;
    tsClients: TTabSheet;
    tsFiles: TTabSheet;
    tsMessages: TTabSheet;
    tsGeneral: TTabSheet;
    procedure bDefaultsClick(Sender: TObject);
    procedure cbHighlightConnectionClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: char);
    procedure SettingsChanged(Sender: TObject);
    procedure bOkClick(Sender: TObject);
    procedure cbLogClick(Sender: TObject);
    procedure shHighlightConnectedMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    FLoading: Boolean;
    FSettings: TSettings;
    FServerStarted: Boolean;
    procedure LoadSettings;
    procedure SaveSettings;
  public
    constructor Create(TheOwner: TComponent; ASettings: TSettings; AServerStarted: Boolean); reintroduce;
  end;

var
  fOptions: TfOptions;

implementation

{$R *.lfm}

constructor TfOptions.Create(TheOwner: TComponent; ASettings: TSettings;
  AServerStarted: Boolean);
begin
  inherited Create(TheOwner);
  FSettings := ASettings;
  pgOptions.ActivePageIndex := 0;
  FServerStarted := AServerStarted;
  FLoading := True;
  LoadSettings;
  FLoading := False;
end;

procedure TfOptions.bDefaultsClick(Sender: TObject);
begin
  if MessageDlg('Restore default settings?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
  begin
    edServerName.Text := 'server';
    cbCredentials.Checked := True;
    cbAutoStart.Checked := False;
    cbHighlightConnection.Checked := True;
    spHighlightInterval.Value := 1;
    shHighlightConnected.Brush.Color := $00E6FFE6;
    shHighlightDisconnected.Brush.Color := $00C6C6FF;
    cbLog.Checked := True;
    cbErrorMessages.Checked := False;
    deDownloads.Directory := AppendPathDelim(GetUserDir) + 'Downloads' + PathDelim;
  end;
end;

procedure TfOptions.FormKeyPress(Sender: TObject; var Key: char);
begin
  if Key = #27 then
    Close;
end;

procedure TfOptions.bOkClick(Sender: TObject);
begin
  if Trim(edServerName.Text) = '' then
  begin
    MessageDlg('Server name cannot be blank!', mtInformation, [mbOk], 0);
    pgOptions.ActivePageIndex := 0;
    edServerName.SetFocus;
    Exit;
  end;
  SaveSettings;
  ModalResult := mrOk;
end;

procedure TfOptions.cbHighlightConnectionClick(Sender: TObject);
begin
  spHighlightInterval.Enabled := cbHighlightConnection.Checked;
  pnHighlightConnected.Enabled := cbHighlightConnection.Checked;
  pnHighlightDisconnected.Enabled := cbHighlightConnection.Checked;
end;

procedure TfOptions.cbLogClick(Sender: TObject);
begin
  cbErrorMessages.Enabled := cbLog.Checked;
  if not cbErrorMessages.Enabled then
    cbErrorMessages.Checked := False;
end;

procedure TfOptions.shHighlightConnectedMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  CD.Color := TShape(Sender).Brush.Color;
  if CD.Execute then
  begin
    TShape(Sender).Brush.Color := CD.Color;
    SettingsChanged(Sender);
  end;
end;

procedure TfOptions.LoadSettings;
begin
  with FSettings do
  begin
    Load;
    edServerName.Text := ServerName;
    edServerName.Enabled := not FServerStarted;
    cbCredentials.Checked := SaveCredentials;
    cbAutoStart.Checked := AutoStart;
    cbHighlightConnection.Checked := HighlightConnection;
    spHighlightInterval.Enabled := cbHighlightConnection.Checked;
    spHighlightInterval.Value := HighlightInterval;
    pnHighlightConnected.Enabled := cbHighlightConnection.Checked;
    shHighlightConnected.Brush.Color := HighlightConnected;
    pnHighlightDisconnected.Enabled := cbHighlightConnection.Checked;
    shHighlightDisconnected.Brush.color := HighlightDisconnected;
    cbLog.Checked := ShowLog;
    cbErrorMessages.Checked := ShowErrorMessages;
    cbErrorMessages.Enabled := cbLog.Checked;
    deDownloads.Directory := DownloadDirectory;
  end;
end;

procedure TfOptions.SaveSettings;
begin
  with FSettings do
  begin
    ServerName := edServerName.Text;
    SaveCredentials := cbCredentials.Checked;
    AutoStart := cbAutoStart.Checked;
    HighlightConnection := cbHighlightConnection.Checked;
    HighlightInterval := spHighlightInterval.Value;
    HighlightConnected := shHighlightConnected.Brush.Color;
    HighlightDisconnected := shHighlightDisconnected.Brush.color;
    ShowLog := cbLog.Checked;
    ShowErrorMessages := cbErrorMessages.Checked;
    DownloadDirectory := deDownloads.Directory;
    Save;
  end;
end;

procedure TfOptions.SettingsChanged(Sender: TObject);
begin
  if FLoading then
    Exit;
  bOk.Enabled := True;
end;

end.

