unit uSettings;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Graphics, LazFileUtils, Laz2_XMLCfg, uTCPSockets;

type
  TSettings = class
  private
    FVersion: Integer;
    FFileName: String;
    FServerName: String;
    FAutoStart: Boolean;
    FBind: TBind;
    FSaveCredentials: Boolean;
    FHighlightConnection: Boolean;
    FHighlightInterval: Integer;
    FHighlightConnected: TColor;
    FHighlightDisconnected: TColor;
    FShowLog: Boolean;
    FShowErrorMessages: Boolean;
    FDownloadDirectory: String;
  public
    constructor Create;
    destructor Destroy; override;
  public
    procedure Load;
    procedure Save;
  public
    property FileName: String read FFileName write FFileName;
    property ServerName: String read FServerName write FServerName;
    property AutoStart: Boolean read FAutoStart write FAutoStart;
    property SaveCredentials: Boolean read FSaveCredentials write FSaveCredentials;
    property ShowErrorMessages: Boolean read FShowErrorMessages write FShowErrorMessages;
    property HighlightConnection: Boolean read FHighlightConnection write FHighlightConnection;
    property HighlightInterval: Integer read FHighlightInterval write FHighlightInterval;
    property HighlightConnected: TColor read FHighlightConnected write FHighlightConnected;
    property HighlightDisconnected: Tcolor read FHighlightDisconnected write FHighlightDisconnected;
    property ShowLog: Boolean read FShowLog write FShowLog;
    property DownloadDirectory: String read FDownloadDirectory write FDownloadDirectory;
    property Bind: TBind read FBind write FBind;
  end;

implementation

uses uCryptoCompression;

const
  InternalVersion = 2;

constructor TSettings.Create;
begin
  FBind := TBind.Create;
end;

destructor TSettings.Destroy;
begin
  FBind.Free;
  inherited Destroy;
end;

procedure TSettings.Load;
var
  XML: TXMLConfig;
  DefDownloadDir: String;
begin
  DefDownloadDir := AppendPathDelim(GetUserDir) + 'Downloads' + PathDelim;
  XML := TXMLConfig.Create(FFileName);
  try
    FVersion := XML.GetValue('Version/Value', 1);
    FServerName := XML.GetValue('ServerName/Value', 'SynapseServer');
    FAutoStart := XML.GetValue('AutoStart/Value', False);
    FSaveCredentials := XML.GetValue('SaveCredentials/Value', True);
    FShowLog := XML.GetValue('ShowLog/Value', True);
    FHighlightConnection := XML.GetValue('HighlightConnection/Value', True);
    FHighlightInterval := XML.GetValue('HighlightInterval/Value', 1);
    FHighlightConnected := XML.GetValue('HighlightConnected/Value', $00E6FFE6);
    FHighlightDisconnected := XML.GetValue('HighlightDisconnected/Value', $00C6C6FF);
    FShowErrorMessages := XML.GetValue('ShowErrorMessages/Value', False);
    FDownloadDirectory := XML.GetValue('DownloadDirectory/Value', DefDownloadDir);
    FBind.FIP := XML.GetValue('BindIP/Value', '0.0.0.0');
    FBind.FPort := XML.GetValue('Port/Value', DefPort);
    if FSaveCredentials then
      FBind.FPass := Decrypt(SHA1Hash(FBind.FPort.ToString), XML.GetValue('Pass/Value', ''));
    FBind.FUser := FServerName;
  finally
    XML.Free;
  end;
end;

procedure TSettings.Save;
var
  XML: TXMLConfig;
  DefDownloadDir: String;
begin
  DefDownloadDir := AppendPathDelim(GetUserDir) + 'Downloads' + PathDelim;
  XML := TXMLConfig.CreateClean(FFileName);
  try
    XML.SetDeleteValue('Version/Value', InternalVersion, 1);
    XML.SetDeleteValue('ServerName/Value', FServerName, 'SynapseServer');
    XML.SetDeleteValue('AutoStart/Value', FAutoStart, False);
    XML.SetDeleteValue('SaveCredentials/Value', FSaveCredentials, True);
    XML.SetDeleteValue('ShowLog/Value', FShowLog, True);
    XML.SetDeleteValue('HighlightConnection/Value', FHighlightConnection, True);
    XML.SetDeleteValue('HighlightInterval/Value', FHighlightInterval, 1);
    XML.SetDeleteValue('HighlightConnected/Value', FHighlightConnected, $00E6FFE6);
    XML.SetDeleteValue('HighlightDisconnected/Value', FHighlightDisconnected, $00C6C6FF);
    XML.SetDeleteValue('ShowErrorMessages/Value', FShowErrorMessages, False);
    XML.SetDeleteValue('DownloadDirectory/Value', FDownloadDirectory, DefDownloadDir);
    XML.SetDeleteValue('BindIP/Value', FBind.FIP, '0.0.0.0');
    XML.SetDeleteValue('Port/Value', FBind.FPort, DefPort);
    if FSaveCredentials then
      XML.SetDeleteValue('Pass/Value', Encrypt(SHA1Hash(FBind.FPort.ToString), FBind.FPass), '');
    XML.Flush;
  finally
    XML.Free;
  end;
end;

end.
