unit uStart;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls,
  Spin, ExtCtrls, Buttons, Menus, uSettings, uTCPSockets;

type
  TfStart = class(TForm)
    bvStart: TBevel;
    bStart: TButton;
    bCancel: TButton;
    edBindIP: TEdit;
    edPass: TEdit;
    lbBindIP: TLabel;
    lbPass: TLabel;
    lbPort: TLabel;
    pnBottom: TPanel;
    spPort: TSpinEdit;
    tmAutoStart: TTimer;
    procedure bStartClick(Sender: TObject);
    procedure edBindIPEnter(Sender: TObject);
    procedure edBindIPKeyPress(Sender: TObject; var Key: char);
    procedure FormKeyPress(Sender: TObject; var Key: char);
    procedure FormShow(Sender: TObject);
    procedure tmAutoStartTimer(Sender: TObject);
  private
    FTCPServer: TTCPServer;
    FSettings: TSettings;
    FAutoStartCnt: Integer;
    procedure EnableDisable(AEnable: Boolean);
    procedure LoadSettings;
    procedure SaveSettings;
  public
    constructor Create(TheOwner: TComponent; ATCPServer: TTCPServer; ASettings: TSettings); reintroduce;
  public
  end;

var
  fStart: TfStart;

implementation

uses uMain;

{$R *.lfm}

constructor TfStart.Create(TheOwner: TComponent; ATCPServer: TTCPServer;
  ASettings: TSettings);
begin
  inherited Create(TheOwner);
  FTCPServer := ATCPServer;
  FSettings := ASettings;
end;

procedure TfStart.FormShow(Sender: TObject);
begin
  LoadSettings;
end;

procedure TfStart.LoadSettings;
begin
  with FSettings do
  begin
    Load;
    edBindIP.Text := FSettings.Bind.FIP;
    spPort.Value := FSettings.Bind.FPort;
    edPass.Text := FSettings.Bind.FPass;
    if FSettings.AutoStart then
    begin
      FAutoStartCnt := 5;
      bStart.Caption := 'Start (' + IntToStr(FAutoStartCnt) + ')';
      tmAutoStart.Enabled := True;
    end;
  end;
end;

procedure TfStart.SaveSettings;
begin
  with FSettings do
  begin
    FSettings.Bind.FIP := edBindIP.Text;
    FSettings.Bind.FPort := spPort.Value;
    FSettings.Bind.FPass := edPass.Text;
    Save;
  end;
end;

procedure TfStart.EnableDisable(AEnable: Boolean);
begin
  edBindIP.Enabled := AEnable;
  lbBindIP.Enabled := AEnable;
  spPort.Enabled := AEnable;
  lbPort.Enabled := AEnable;
  edPass.Enabled := AEnable;
  lbPass.Enabled := AEnable;
  bStart.Enabled := AEnable;
  bCancel.Enabled := AEnable;
end;

procedure TfStart.bStartClick(Sender: TObject);
var
  LogStr: String;
begin
  if Trim(edBindIP.Text) = '' then
  begin
    edBindIp.SetFocus;
    MessageDlg('Please enter a valid ip or host.', mtInformation, [mbOk], 0);
    Exit;
  end;
  if (tmAutoStart.Enabled) then
  begin
    tmAutoStart.Enabled := False;
    bStart.Caption := 'Start';
  end;

  EnableDisable(False);
  Self.Invalidate;
  Application.ProcessMessages;
  SaveSettings;
  FTCPServer.ShowErrorMessages := FSettings.ShowErrorMessages;
  FTCPServer.DownloadDirectory := FSettings.DownloadDirectory;
  FTCPServer.Start(FSettings.Bind);
  EnableDisable(True);
  if FTCPServer.Active then
    ModalResult := mrOk
  else
  begin
    LogStr := 'Cannot start server.';
    fMain.sb.Tag := 2;
    fMain.sb.Panels[0].Text := LogStr;
    MessageDlg(LogStr + ' Error message: ' + sLineBreak + '"' + FTCPServer.LastErrMsg + '(' + IntToStr(FTCPServer.LastErrCode) + ')"', mtInformation, [mbOk], 0);
  end;
end;

procedure TfStart.tmAutoStartTimer(Sender: TObject);
begin
  Dec(FAutoStartCnt);
  bStart.Caption := 'Start (' + IntToStr(FAutoStartCnt) + ')';
  if FAutoStartCnt = 0 then
  begin
    tmAutoStart.Enabled := False;
    bStart.Caption := 'Start';
    bStartClick(bStart);
  end;
end;

procedure TfStart.edBindIPEnter(Sender: TObject);
begin
  if (FSettings.AutoStart) and (tmAutoStart.Enabled) then
  begin
    tmAutoStart.Enabled := False;
    bStart.Caption := 'Start';
  end;
end;

procedure TfStart.edBindIPKeyPress(Sender: TObject; var Key: char);
begin
  if Key = #13 then
  begin
    Key := #0;
    bStartClick(bStart);
  end;
end;

procedure TfStart.FormKeyPress(Sender: TObject; var Key: char);
begin
  if Key = #27 then
    Close;
end;

end.

