unit uCommunication;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ComCtrls, ExtCtrls,
  Menus, FileUtil, laz.VirtualTrees, uFileTransfer, uMessages, uMain, uTCPSockets;

type
  TfCommunication = class(TForm)
    miClear: TMenuItem;
    pgCommunication: TPageControl;
    pmMsg: TPopupMenu;
    spFiles: TSplitter;
    spMsg: TSplitter;
    tsFiles: TTabSheet;
    tsMessages: TTabSheet;
    VSTClients: TLazVirtualStringTree;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure pgCommunicationChange(Sender: TObject);
    procedure VSTClientsAddToSelection(Sender: TBaseVirtualTree;
      Node: PVirtualNode);
    procedure VSTClientsBeforeCellPaint(Sender: TBaseVirtualTree;
      TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
      CellPaintMode: TVTCellPaintMode; CellRect: TRect; var ContentRect: TRect);
    procedure VSTClientsCompareNodes(Sender: TBaseVirtualTree; Node1,
      Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
    procedure VSTClientsFocusChanged(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex);
    procedure VSTClientsFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
    procedure VSTClientsGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType; var CellText: String);
    procedure VSTClientsHeaderClick(Sender: TVTHeader; HitInfo: TVTHeaderHitInfo);
    procedure VSTClientsPaintText(Sender: TBaseVirtualTree;
      const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
      TextType: TVSTTextType);
    procedure VSTClientsRemoveFromSelection(Sender: TBaseVirtualTree;
      Node: PVirtualNode);
  private
    FFileTransfer: TfrFileTransfer;
    FMessages: TfrMessages;
    FSortDir: laz.VirtualTrees.TSortDirection;
    procedure DoOnChanged(Sender: TObject; AType: Byte);
    procedure DoOnSendFiles(Sender: TObject; AFiles: TStrings);
    procedure DoOnSendMessage(Sender: TObject; AMsg: String);
  public
    procedure AddClientNode(ANode: PVirtualNode; AData: PData);
    procedure DeleteClientNode(ANode: PVirtualNode);
    procedure ChangeClientStatus(ANode: PVirtualNode; AClientStatus: TClientStatus);
  public
    property FileTransfer: TfrFileTransfer read FFileTransfer;
    property Messages: TfrMessages read FMessages;
  end;

var
  fCommunication: TfCommunication;

implementation

uses uFunctions;

{$R *.lfm}

procedure TfCommunication.FormCreate(Sender: TObject);
begin
  Self.Width := fMain.Width + 150;
  Self.Height := fMain.Height - 20;
  VSTClients.Parent := Self;
  VSTClients.NodeDataSize := SizeOf(TData);

  FFileTransfer := TfrFileTransfer.Create(Self);
  FFileTransfer.Parent := tsFiles;
  FFileTransfer.Align := alClient;
  FFileTransfer.InitializeFrame(2);
  FFileTransfer.OnSendFiles := @DoOnSendFiles;
  FFileTransfer.OnChanged := @DoOnChanged;

  FMessages := TfrMessages.Create(nil);
  FMessages.Parent := tsMessages;
  FMessages.Align := alClient;
  FMessages.InitializeFrame(2);
  FMessages.OnChanged := @DoOnChanged;
  FMessages.OnSendMessage := @DoOnSendMessage;

  pgCommunication.ActivePageIndex := 0;
  pgCommunicationChange(pgCommunication);
end;

procedure TfCommunication.FormDestroy(Sender: TObject);
begin
  FFileTransfer.FinalizeFrame;
  FFileTransfer.Free;
  FMessages.FinalizeFrame;
  FMessages.Free;
end;

procedure TfCommunication.pgCommunicationChange(Sender: TObject);
begin
  case pgCommunication.ActivePageIndex of
    0: VSTClients.Parent := FFileTransfer.pnTree;
    1: VSTClients.Parent := FMessages.pnTree;
  end;
end;

procedure TfCommunication.DoOnChanged(Sender: TObject; AType: Byte);
begin
  if not Self.Visible then
  begin
    Self.Visible := True;
    pgCommunication.ActivePageIndex := AType;
  end;
end;

procedure TfCommunication.DoOnSendMessage(Sender: TObject; AMsg: String);
var
  Node: PVirtualNode;
  Data: PData;
begin
  Node := VSTClients.GetFirstSelected;
  while Node <> nil do
  begin
    Data := VSTClients.GetNodeData(Node);
    if (Data <> nil) and (Data^.FClient <> nil) and (Data^.FClient.Connected) then
      Data^.FClient.SendMessage(AMsg, []);
    Node := VSTClients.GetNextSelected(Node);
  end;
end;

procedure TfCommunication.DoOnSendFiles(Sender: TObject; AFiles: TStrings);
var
  I: Integer;
  ClientNode: PVirtualNode;
  ClientData: PData;
  FileNode: PVirtualNode;
  FileData: PFileData;
begin
  for I := 0 to AFiles.Count - 1 do
  begin
    ClientNode := VSTClients.GetFirstSelected;
    while ClientNode <> nil do
    begin
      ClientData := VSTClients.GetNodeData(ClientNode);
      if (ClientData <> nil) and (ClientData^.FClient <> nil) and (ClientData^.FClient.Connected) then
      begin
        FileNode := FFileTransfer.VSTFiles.AddChild(nil);
        FileData := FFileTransfer.VSTFiles.GetNodeData(FileNode);
        FileData^.FFileName := AFiles[I];
        FileData^.FProgress := 0;
        FileData^.FProgressStr := 'Please wait...';
        FileData^.FSizeStr := FormatSize(0);
        FileData^.FSpeedStr := FormatSpeed(0);
        FileData^.FRemainingStr := 'unknown';
        FileData^.FTotalSize := FileSize(AFiles[I]);
        FileData^.FTotalSizeStr := FormatSize(FileSize(AFiles[I]));
        ClientData^.FClient.SendFile(AFiles[I], PVirtualNode(FileNode));
        Application.ProcessMessages;
      end;
      ClientNode := VSTClients.GetNextSelected(ClientNode);
    end;
  end;
end;

{$Region 'Connected Clients'}
procedure TfCommunication.AddClientNode(ANode: PVirtualNode; AData: PData);
var
  Data: PData;
  Node: PVirtualNode;
begin
  VSTClients.BeginUpdate;
  try
    Node := VSTClients.AddChild(nil);
    Data := VSTClients.GetNodeData(Node);
    Data^ := TData.Create;
    Data^.FLocalUser := AData^.FLocalUser;
    Data^.FRemoteUser := AData^.FRemoteUser;
    Data^.FCountDown := AData^.FCountDown;
    Data^.FClient := AData^.FClient;
    Data^.FLinkNode := ANode;
    AData^.FLinkNode := Node;
    VSTClients.Sort(nil, 0, FSortDir);
  finally
    VSTClients.EndUpdate;
  end;
end;

procedure TfCommunication.DeleteClientNode(ANode: PVirtualNode);
var
  Data: PData;
  Node: PVirtualNode;
begin
  Data := VSTClients.GetNodeData(ANode);
  if (Data <> nil) and (Data^.FLinkNode <> nil) then
    Data^.FLinkNode := nil;
  VSTClients.DeleteNode(ANode);
  Node := VSTClients.GetFirstSelected;
  if Node <> nil then
    VSTClients.FocusedNode := Node;
end;

procedure TfCommunication.ChangeClientStatus(ANode: PVirtualNode;
  AClientStatus: TClientStatus);
var
  Data: PData;
begin
  Data := VSTClients.GetNodeData(ANode);
  Data^.FClientStatus := AClientStatus;
  VSTClients.ReinitNode(ANode, False);
  VSTClients.RepaintNode(ANode);
end;

procedure TfCommunication.VSTClientsGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: String);
var
  Data: PData;
begin
  Data := Sender.GetNodeData(Node);
  if TData(Data^) = nil then
    Exit;
  case Column of
    0: CellText := Data^.FLocalUser;
  end;
end;

procedure TfCommunication.VSTClientsCompareNodes(Sender: TBaseVirtualTree; Node1,
  Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
var
  Data1: PData;
  Data2: PData;
begin
  Data1 := Sender.GetNodeData(Node1);
  Data2 := Sender.GetNodeData(Node2);
  if Column = 0 then
    Result := CompareText(Data1^.FRemoteUser, Data2^.FRemoteUser);
end;

procedure TfCommunication.VSTClientsHeaderClick(Sender: TVTHeader;
  HitInfo: TVTHeaderHitInfo);
begin
  if (HitInfo.Column > 0) then
    Exit;
  if HitInfo.Button = mbLeft then
  begin
    with Sender, Treeview do
    begin
      if SortDirection = laz.VirtualTrees.sdAscending then
        SortDirection := laz.VirtualTrees.sdDescending
      else
        SortDirection := laz.VirtualTrees.sdAscending;
      FSortDir := SortDirection;
      Sort(nil, 0, SortDirection);
    end;
  end;
end;

procedure TfCommunication.VSTClientsBeforeCellPaint(Sender: TBaseVirtualTree;
  TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  CellPaintMode: TVTCellPaintMode; CellRect: TRect; var ContentRect: TRect);
begin
  if CellPaintMode = cpmPaint then
  begin
    if (Node = Sender.FocusedNode) then
      TargetCanvas.Brush.Color := clHighlight
    else
      TargetCanvas.Brush.Color := clWindow;
    TargetCanvas.FillRect(CellRect);
  end;
end;

procedure TfCommunication.VSTClientsPaintText(Sender: TBaseVirtualTree;
  const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType);
begin
  if Node = Sender.FocusedNode then
    TargetCanvas.Font.Color := clHighlightText
  else
    TargetCanvas.Font.Color := Sender.Font.Color;
end;

procedure TfCommunication.VSTClientsFocusChanged(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex);
var
  Data: PData;
begin
  Data := VSTClients.GetNodeData(Node);
  if (Data <> nil) and (Data^.FLinkNode <> nil) then
    fMain.VSTCon.FocusedNode := Data^.FLinkNode;
end;

procedure TfCommunication.VSTClientsAddToSelection(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
var
  Data: PData;
begin
  Data := VSTClients.GetNodeData(Node);
  if (Data <> nil) and (Data^.FLinkNode <> nil) then
    fMain.VSTCon.Selected[Data^.FLinkNode] := True;
end;

procedure TfCommunication.VSTClientsRemoveFromSelection(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
var
  Data: PData;
begin
  Data := VSTClients.GetNodeData(Node);
  if (Data <> nil) and (Data^.FLinkNode <> nil) then
    fMain.VSTCon.Selected[Data^.FLinkNode] := False;
end;

procedure TfCommunication.VSTClientsFreeNode(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
var
  Data: PData;
begin
  Data := VSTClients.GetNodeData(Node);
  Data^.Free;
end;
{$EndRegion}

end.

