unit uMain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ComCtrls,
  ExtCtrls, Spin, Menus, ActnList, Math, laz.VirtualTrees, uTCPSockets;

const
  Title = 'Synapse server stress test';
  Version = '2.0';
  SynTrunkRev = 267;

type
  TClientStatus = (csConnecting, csConnected, csCounting, csDisconnecting, csDisconnected);

  PData = ^TData;
  TData = class
    FLocalUser: String;
    FRemoteUser: String;
    FCountDown: Integer;
    FLinkNode: PVirtualNode;
    FClientStatus: TClientStatus;
    FClient: TTCPClient;
  end;

  TfMain = class(TForm)
    bGo: TButton;
    cbDisconnect: TCheckBox;
    edIP: TEdit;
    edPass: TEdit;
    gbConnection: TGroupBox;
    gbOptions: TGroupBox;
    lbClients: TLabel;
    lbDisconnect: TLabel;
    lbIP: TLabel;
    lbPass: TLabel;
    lbPort: TLabel;
    miOpenSelected: TMenuItem;
    miDisconnect: TMenuItem;
    pnButtons: TPanel;
    pmClients: TPopupMenu;
    spClients: TSpinEdit;
    spDisconnect: TSpinEdit;
    spPort: TSpinEdit;
    sb: TStatusBar;
    tmTree: TTimer;
    tmClose: TTimer;
    VSTCon: TLazVirtualStringTree;
    procedure bGoClick(Sender: TObject);
    procedure cbUseColorsClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure miOpenSelectedClick(Sender: TObject);
    procedure miDisconnectClick(Sender: TObject);
    procedure tmCloseTimer(Sender: TObject);
    procedure tmTreeTimer(Sender: TObject);
    procedure VSTConAddToSelection(Sender: TBaseVirtualTree; Node: PVirtualNode
      );
    procedure VSTConBeforeCellPaint(Sender: TBaseVirtualTree;
      TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
      CellPaintMode: TVTCellPaintMode; CellRect: TRect; var ContentRect: TRect);
    procedure VSTConCompareNodes(Sender: TBaseVirtualTree; Node1,
      Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
    procedure VSTConDblClick(Sender: TObject);
    procedure VSTConFocusChanged(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex);
    procedure VSTConFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
    procedure VSTConGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType; var CellText: String);
    procedure VSTConHeaderClick(Sender: TVTHeader; HitInfo: TVTHeaderHitInfo);
    procedure VSTConMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure VSTConPaintText(Sender: TBaseVirtualTree;
      const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
      TextType: TVSTTextType);
    procedure VSTConRemoveFromSelection(Sender: TBaseVirtualTree;
      Node: PVirtualNode);
  private
    FBusyClose: Boolean;
    FBusyDisconnect: Boolean;
    FClientCnt: Integer;
    FUniqueID: Integer;
    FClosing: Boolean;
    FSortCol: Integer;
    FSortDir: laz.VirtualTrees.TSortDirection;
    function Connect(out AErrMsg: String; out AErrCode: Integer): Boolean;
    procedure ChangeStatus(ANode: PVirtualNode; AClientStatus: TClientStatus);
    procedure DoOnConnected(Sender: TObject; AConData: TConData; AConCnt: Integer = 0);
    procedure DoOnDisconnected(Sender: TObject; AConData: TConData; AConCnt: Integer = 0; AByUser: Boolean = False);
  public

  end;

var
  fMain: TfMain;

implementation
uses uCommunication;

{$R *.lfm}

procedure TfMain.FormCreate(Sender: TObject);
begin
  FClientCnt := 0;
  VSTCon.NodeDataSize := SizeOf(TData);
  FSortCol := 0;
  FSortDir := laz.VirtualTrees.sdAscending;
  fCommunication := TfCommunication.Create(Self);
  Self.Width := 720;
  Self.Height := 470;
  Randomize;
  tmTree.Enabled := True;
  FUniqueID := Random(10000) + 1;
  Caption := Title + '(UniqueID: ' + IntToStr(FUniqueID) + ')';
  Application.Title := Title + ' ' + Version;
end;

procedure TfMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := MessageDlg('Are you sure you wish to quit?', mtConfirmation, [mbYes, mbNo], 0) = mrYes;
  if not CanClose then
    Exit;
  CanClose := False;
  tmTree.Enabled := False;
  FClosing := True;
  Caption := 'Closing. Please wait...';
  fMain.Enabled := False;
  tmClose.Enabled := True;
end;

procedure TfMain.bGoClick(Sender: TObject);
var
  I: Integer;
  ErrMsg: String;
  ErrCode: Integer;
begin
  bGo.Enabled := False;
  Caption := 'Connecting. Please wait...';
  Application.ProcessMessages;
  try
    for I := 1 to spClients.Value do
    begin
      if FClosing then
        Break;
      Inc(FClientCnt);
      if not Connect(ErrMsg, ErrCode) then
      begin
        Dec(FClientCnt);
        MessageDlg('Cannot connect to server!' + sLineBreak + ErrMsg + '(' + ErrCode.ToString + ')', mtInformation, [mbOk], 0);
        Break;
      end;
      sb.Panels[0].Text := 'Connections: ' + FClientCnt.ToString;
      Caption := 'Connecting. Please wait...(' + I.ToString + ')';
      Application.ProcessMessages;
    end;
  finally
    bGo.Enabled := True;
    Caption := Title + '(UniqueID: ' + IntToStr(FUniqueID) + ')';
  end;
end;

function TfMain.Connect(out AErrMsg: String; out AErrCode: Integer): Boolean;
var
  Connection: TConnection;
  Data: PData;
  Node: PVirtualNode;
  User: String;
begin
  Result := True;
  Connection := TConnection.Create;
  try
    User := 'user(' + Format('%.*d', [4, FUniqueID]) + ')_' + Format('%.*d', [4, FClientCnt]) ;
    Node := VSTCon.AddChild(nil);
    Data := VSTCon.GetNodeData(Node);
    Data^ := TData.Create;
    Data^.FLocalUser := User;
    Data^.FRemoteUser := '';
    Data^.FCountDown := RandomRange(5, spDisconnect.Value + 1);
    Data^.FClient := TTCPClient.Create;
    Data^.FClient.ShowErrorMessages := True;
    Data^.FClient.OnConnected := @DoOnConnected;
    Data^.FClient.OnDisconnected := @DoOnDisconnected;
    Data^.FClient.OnSentFile := @fCommunication.FileTransfer.DoOnSentFile;
    Data^.FClient.OnRecvFile := @fCommunication.FileTransfer.DoOnRecvFile;
    Data^.FClient.OnFileProgress := @fCommunication.FileTransfer.DoOnFileProgress;
    Data^.FClient.OnSentMessage := @fCommunication.Messages.DoOnSentMessage;
    Data^.FClient.OnRecvMessage := @fCommunication.Messages.DoOnRecvMessage;
    ChangeStatus(Node, csConnecting);
    Connection.FIP := edIP.Text;
    Connection.FPort := spPort.Value;
    Connection.FUser := User;
    Connection.FPass := edPass.Text;
    Connection.FData := Node;
    Data^.FClient.Connect(Connection);
    Result := Data^.FClient.Connected;
    if not Result then
    begin
      AErrMsg := Data^.FClient.LastErrMsg;
      AErrCode := Data^.FClient.LastErrCode;
      VSTCon.DeleteNode(Node);
    end;
  finally
    Connection.Free;
  end;
end;

procedure TfMain.ChangeStatus(ANode: PVirtualNode; AClientStatus: TClientStatus);
var
  Data: PData;
begin
  Data := VSTCon.GetNodeData(ANode);
  Data^.FClientStatus := AClientStatus;
  VSTCon.ReinitNode(ANode, False);
  VSTCon.RepaintNode(ANode);
end;

procedure TfMain.cbUseColorsClick(Sender: TObject);
begin
  VSTCon.Repaint;
end;

procedure TfMain.tmCloseTimer(Sender: TObject);
var
  ConCnt: Integer;
  Data: PData;
  Node: PVirtualNode;
begin
  if FBusyClose then
    Exit;
  FBusyClose := True;
  try
    ConCnt := 0;
    Node := VSTCon.GetFirst;
    while Node <> nil do
    begin
      Data := VSTCon.GetNodeData(Node);
      if Data^.FClient.Connected then
      begin
        Inc(ConCnt);
        ChangeStatus(Node, csDisconnecting);
        if Data^.FClient.CC <> nil then
          Data^.FClient.CC.ForceQuit := True;
      end
      else
        ChangeStatus(Node, csDisconnected);
      Node := VSTCon.GetNext(Node);
    end;
  finally
    FBusyClose := False;
  end;
  tmClose.Enabled := ConCnt > 0;
  if not tmClose.Enabled then
    Application.Terminate;
end;

procedure TfMain.tmTreeTimer(Sender: TObject);
var
  I: Integer;
  Len: Integer;
  Data: PData;
  Node: PVirtualNode;
  Nodes: array of PVirtualNode;
begin
  if (FBusyDisconnect) or (FClosing) then
    Exit;
  FBusyDisconnect := True;
  try
    Nodes := nil;
    Node := VSTCon.GetFirst;
    while Node <> nil do
    begin
      Data := VSTCon.GetNodeData(Node);
      if (Data^.FClient.Connected) and (Data^.FCountDown > 0) and (cbDisconnect.Checked) then
      begin
        Dec(Data^.FCountDown);
        if Data^.FCountDown = 0 then
        begin
          ChangeStatus(Node, csDisconnecting);
          if (Data^.FClient.CC <> nil) then
            Data^.FClient.CC.ForceQuit := True;
        end
        else
          ChangeStatus(Node, csCounting);
      end
      else
        case Data^.FClientStatus of
          csDisconnecting: ChangeStatus(Node, csDisconnected);
          csDisconnected:
            begin
              Len := Length(Nodes);
              SetLength(Nodes, Len + 1);
              Nodes[Len] := Node;
            end;
        end;
      Node := VSTCon.GetNext(Node);
    end;
    for I := High(Nodes) downto Low(Nodes) do
    begin
      Data := VSTCon.GetNodeData(Nodes[I]);
      fCommunication.DeleteClientNode(Data^.FLinkNode);
      VSTCon.DeleteNode(Nodes[I]);
      Node := VSTCon.GetFirstSelected;
      if Node <> nil then
        VSTCon.FocusedNode := Node;
    end;
  finally
    FBusyDisconnect := False;
  end;
end;

procedure TfMain.DoOnConnected(Sender: TObject; AConData: TConData;
  AConCnt: Integer);
var
  Data: PData;
  Node: PVirtualNode;
begin
  if FClosing then
    Exit;
  Node := PVirtualNode(AConData.FData);
  if Node <> nil then
  begin
    Data := VSTCon.GetNodeData(Node);
    Data^.FRemoteUser := AConData.FRemoteUser;
    ChangeStatus(Node, csConnected);
    fCommunication.AddClientNode(Node, Data);
    if VSTCon.FocusedNode = nil then
    begin
      VSTCon.FocusedNode := Node;
      VSTCon.Selected[Node] := True;
    end;
  end;
end;

procedure TfMain.DoOnDisconnected(Sender: TObject; AConData: TConData;
  AConCnt: Integer = 0; AByUser: Boolean = False);
var
  Node: PVirtualNode;
begin
  if FClosing then
    Exit;
  Node := PVirtualNode(AConData.FData);
  if Node <> nil then
  begin
    ChangeStatus(Node, csDisconnecting);
    VSTCon.ReinitNode(Node, False);
    VSTCon.RepaintNode(Node);
  end;
end;

procedure TfMain.miDisconnectClick(Sender: TObject);
var
  Data: PData;
  Node: PVirtualNode;
begin
  if FClosing then
    Exit;
  VSTCon.BeginUpdate;
  try
    Node := VSTCon.GetFirst;
    while Node <> nil do
    begin
      if VSTCon.Selected[Node] then
      begin
        Data := VSTCon.GetNodeData(Node);
        if Data^.FClient.Connected then
          Data^.FClient.CC.ForceQuit := True;
      end;
      Node := VSTCon.GetNext(Node);
    end;
  finally
    VSTCon.EndUpdate;
  end;
end;

procedure TfMain.miOpenSelectedClick(Sender: TObject);
begin
  if FClosing then
    Exit;
  if fCommunication.Visible then
    fCommunication.BringToFront
  else
    fCommunication.Show;
end;

procedure TfMain.VSTConFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
var
  Data: PData;
begin
  Data := VSTCon.GetNodeData(Node);
  if TData(Data^) = nil then
    Exit;
  Data^.FClient.Free;
  Data^.Free;
end;

procedure TfMain.VSTConBeforeCellPaint(Sender: TBaseVirtualTree;
  TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  CellPaintMode: TVTCellPaintMode; CellRect: TRect; var ContentRect: TRect);
begin
  if CellPaintMode = cpmPaint then
  begin
    if (Node = Sender.FocusedNode) then
      TargetCanvas.Brush.Color := clHighlight
    else
      TargetCanvas.Brush.Color := clWindow;
    TargetCanvas.FillRect(CellRect);
  end;
end;

procedure TfMain.VSTConMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  P: TPoint;
  Node: PVirtualNode;
begin
  if FClosing then
    Exit;
  if Button = mbRight then
  begin
    Node := VSTCon.GetNodeAt(X, Y);
    miDisconnect.Enabled := (Node <> nil) and (VSTCon.Selected[Node]);
    miOpenSelected.Enabled := VSTCon.RootNodeCount > 0;
    P.X := X;
    P.Y := Y;
    P := VSTCon.ClientToScreen(P);
    pmClients.PopUp(P.X, P.Y);
  end;
end;

procedure TfMain.VSTConPaintText(Sender: TBaseVirtualTree;
  const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType);
begin
  if Node = Sender.FocusedNode then
    TargetCanvas.Font.Color := clHighlightText
  else
    TargetCanvas.Font.Color := Sender.Font.Color;
end;

procedure TfMain.VSTConGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
  Column: TColumnIndex; TextType: TVSTTextType; var CellText: String);
var
  Data: PData;
begin
  Data := VSTCon.GetNodeData(Node);
  if TData(Data^) = nil then
    Exit;
  case Column of
    0: CellText := Data^.FLocalUser;
    1: case Data^.FClientStatus of
         csConnecting: CellText := 'connecting';
         csConnected, csCounting: CellText := 'connected';
         csDisconnecting: CellText := 'disconnecting';
         csDisconnected: CellText := 'disconnected';
       end;
    2: CellText := Data^.FCountDown.ToString;
  end;
end;

procedure TfMain.VSTConCompareNodes(Sender: TBaseVirtualTree; Node1,
  Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
var
  Data1: PData;
  Data2: PData;
begin
  Data1 := Sender.GetNodeData(Node1);
  Data2 := Sender.GetNodeData(Node2);
  case Column of
    0: Result := CompareText(Data1^.FLocalUser, Data2^.FLocalUser);
    1: Result := Ord(Data1^.FClientStatus) - Ord(Data2^.FClientStatus);
    2: Result := Data1^.FCountDown - Data2^.FCountDown;
  end;
end;

procedure TfMain.VSTConDblClick(Sender: TObject);
begin
  if FClosing then
    Exit;
  miOpenSelectedClick(miOpenSelected);
end;

procedure TfMain.VSTConFocusChanged(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex);
var
  Data: PData;
begin
  if FClosing then
    Exit;
  Data := VSTCon.GetNodeData(Node);
  if (Data <> nil) and (Data^.FLinkNode <> nil) then
    fCommunication.VSTClients.FocusedNode:= Data^.FLinkNode;
end;

procedure TfMain.VSTConAddToSelection(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
var
  Data: PData;
begin
  if FClosing then
    Exit;
  Data := VSTCon.GetNodeData(Node);
  if (Data <> nil) and (Data^.FLinkNode <> nil) then
    fCommunication.VSTClients.Selected[Data^.FLinkNode] := True;
end;

procedure TfMain.VSTConRemoveFromSelection(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
var
  Data: PData;
begin
  if FClosing then
    Exit;
  Data := VSTCon.GetNodeData(Node);
  if (Data <> nil) and (Data^.FLinkNode <> nil) then
    fCommunication.VSTClients.Selected[Data^.FLinkNode] := False;
end;

procedure TfMain.VSTConHeaderClick(Sender: TVTHeader; HitInfo: TVTHeaderHitInfo);
begin
  if (HitInfo.Column > 2) or (FClosing) then
    Exit;
  if HitInfo.Button = mbLeft then
  begin
    with Sender, Treeview do
    begin
      if (SortColumn = NoColumn) or (SortColumn <> HitInfo.Column) then
      begin
        SortColumn    := HitInfo.Column;
        SortDirection := laz.VirtualTrees.sdAscending;
      end
      else
      begin
        if SortDirection = laz.VirtualTrees.sdAscending then
          SortDirection := laz.VirtualTrees.sdDescending
        else
          SortDirection := laz.VirtualTrees.sdAscending;
        FSortDir := SortDirection;
      end;
      SortTree(SortColumn, SortDirection, False);
      FSortCol := Sender.SortColumn;
    end;
  end;
end;

end.

